@@ -22,11 +22,14 @@ import java.io.BufferedWriter;
 import java.io.File;
 import java.io.FileReader;
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
 import java.io.PrintStream;
+import java.io.Reader;
+import java.nio.file.Files;
 
 import org.apache.tools.ant.ProjectComponent;
-import org.apache.tools.ant.util.FileUtils;
 
 /**
  * Class representing an email message.
@@ -39,12 +42,12 @@ public class Message extends ProjectComponent {
     private String mimeType = "text/plain";
     private boolean specified = false;
     private String charset = null;
+    private String inputEncoding;
 
     /** Creates a new empty message  */
     public Message() {
     }
 
-
     /**
      * Creates a new message based on the given string
      *
@@ -54,7 +57,6 @@ public class Message extends ProjectComponent {
         addText(text);
     }
 
-
     /**
      * Creates a new message using the contents of the given file.
      *
@@ -64,7 +66,6 @@ public class Message extends ProjectComponent {
         messageSource = file;
     }
 
-
     /**
      * Adds a textual part of the message
      *
@@ -74,7 +75,6 @@ public class Message extends ProjectComponent {
         buffer.append(text);
     }
 
-
     /**
      * Sets the source file of the message
      *
@@ -84,7 +84,6 @@ public class Message extends ProjectComponent {
         this.messageSource = src;
     }
 
-
     /**
      * Sets the content type for the message
      *
@@ -95,7 +94,6 @@ public class Message extends ProjectComponent {
         specified = true;
     }
 
-
     /**
      * Returns the content type
      *
@@ -105,35 +103,28 @@ public class Message extends ProjectComponent {
         return mimeType;
     }
 
-
     /**
      * Prints the message onto an output stream
      *
      * @param ps The print stream to write to
      * @throws IOException if an error occurs
      */
-    public void print(PrintStream ps)
-         throws IOException {
+    @SuppressWarnings("resource")
+    public void print(PrintStream ps) throws IOException {
         // We need character encoding aware printing here.
         // So, using BufferedWriter over OutputStreamWriter instead of PrintStream
         BufferedWriter out = null;
         try {
-            out
-                = charset != null ? new BufferedWriter(new OutputStreamWriter(ps, charset))
-                : new BufferedWriter(new OutputStreamWriter(ps));
+            out = charset == null ? new BufferedWriter(new OutputStreamWriter(ps))
+                    : new BufferedWriter(new OutputStreamWriter(ps, charset));
             if (messageSource != null) {
                 // Read message from a file
-                FileReader freader = new FileReader(messageSource);
-
-                try {
-                    BufferedReader in = new BufferedReader(freader);
-                    String line = null;
+                try (BufferedReader in = new BufferedReader(getReader(messageSource))) {
+                    String line;
                     while ((line = in.readLine()) != null) {
                         out.write(getProject().replaceProperties(line));
                         out.newLine();
                     }
-                } finally {
-                    freader.close();
                 }
             } else {
                 out.write(getProject().replaceProperties(buffer.substring(0)));
@@ -145,7 +136,6 @@ public class Message extends ProjectComponent {
         }
     }
 
-
     /**
      * Returns true if the mimeType has been set.
      *
@@ -162,8 +152,9 @@ public class Message extends ProjectComponent {
      * @since Ant 1.6
      */
     public void setCharset(String charset) {
-      this.charset = charset;
+        this.charset = charset;
     }
+
     /**
      * Returns the charset of mail message.
      *
@@ -171,7 +162,29 @@ public class Message extends ProjectComponent {
      * @since Ant 1.6
      */
     public String getCharset() {
-      return charset;
+        return charset;
+    }
+
+    /**
+     * Sets the encoding to expect when reading the message from a file.
+     * <p>Will be ignored if the message has been specified inline.</p>
+     * @param encoding the name of the charset used
+     * @since Ant 1.9.4
+     */
+    public void setInputEncoding(String encoding) {
+        this.inputEncoding = encoding;
+    }
+
+    private Reader getReader(File f) throws IOException {
+        if (inputEncoding != null) {
+            InputStream fis = Files.newInputStream(f.toPath());
+            try {
+                return new InputStreamReader(fis, inputEncoding);
+            } catch (IOException ex) {
+                fis.close();
+                throw ex;
+            }
+        }
+        return new FileReader(f);
     }
 }
-
