@@ -18,17 +18,18 @@
 
 package org.apache.tools.ant.taskdefs.optional.ssh;
 
+import java.io.BufferedWriter;
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.StringReader;
+import java.nio.file.Files;
 
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
@@ -218,7 +219,7 @@ public class SSHExec extends SSHBase {
      *      will be stored.
      * @since Apache Ant 1.9.4
      */
-    public void setErrorproperty (final String property) {
+    public void setErrorproperty(final String property) {
         errorProperty = property;
     }
 
@@ -235,6 +236,7 @@ public class SSHExec extends SSHBase {
 
     /**
      * Whether a pseudo-tty should be allocated.
+     * @param b boolean
      * @since Apache Ant 1.8.3
      */
     public void setUsePty(final boolean b) {
@@ -252,8 +254,9 @@ public class SSHExec extends SSHBase {
     }
 
     /**
-     * If suppressSystemOut is <code>true</code>, output will not be sent to System.out<br/>
-     * If suppressSystemOut is <code>false</code>, normal behavior
+     * If suppressSystemOut is <code>true</code>, output will not be sent to System.out,
+     * if suppressSystemOut is <code>false</code>, normal behavior
+     * @param suppressSystemOut boolean
      * @since Ant 1.9.0
      */
     public void setSuppressSystemOut(final boolean suppressSystemOut) {
@@ -261,8 +264,9 @@ public class SSHExec extends SSHBase {
     }
 
     /**
-     * If suppressSystemErr is <code>true</code>, output will not be sent to System.err<br/>
-     * If suppressSystemErr is <code>false</code>, normal behavior
+     * If suppressSystemErr is <code>true</code>, output will not be sent to System.err,
+     * if suppressSystemErr is <code>false</code>, normal behavior
+     * @param suppressSystemErr boolean
      * @since Ant 1.9.4
      */
     public void setSuppressSystemErr(final boolean suppressSystemErr) {
@@ -295,18 +299,16 @@ public class SSHExec extends SSHBase {
             + (inputProperty != null ? 1 : 0)
             + (inputString != null ? 1 : 0);
         if (numberOfInputs > 1) {
-            throw new BuildException("You can't specify more than one of"
-                                     + " inputFile, inputProperty and"
-                                     + " inputString.");
+            throw new BuildException(
+                "You can't specify more than one of inputFile, inputProperty and inputString.");
         }
         if (inputFile != null && !inputFile.exists()) {
-            throw new BuildException("The input file "
-                                     + inputFile.getAbsolutePath()
-                                     + " does not exist.");
+            throw new BuildException("The input file %s does not exist.",
+                inputFile.getAbsolutePath());
         }
 
         Session session = null;
-        final StringBuffer output = new StringBuffer();
+        final StringBuilder output = new StringBuilder();
         try {
             session = openSession();
             /* called once */
@@ -314,32 +316,27 @@ public class SSHExec extends SSHBase {
                 log("cmd : " + command, Project.MSG_INFO);
                 executeCommand(session, command, output);
             } else { // read command resource and execute for each command
-                try {
-                    final BufferedReader br = new BufferedReader(
-                            new InputStreamReader(commandResource.getInputStream()));
-                    String cmd;
-                    while ((cmd = br.readLine()) != null) {
+                try (final BufferedReader br = new BufferedReader(
+                    new InputStreamReader(commandResource.getInputStream()))) {
+                    final Session s = session;
+                    br.lines().forEach(cmd -> {
                         log("cmd : " + cmd, Project.MSG_INFO);
                         output.append(cmd).append(" : ");
-                        executeCommand(session, cmd, output);
+                        executeCommand(s, cmd, output);
                         output.append("\n");
-                    }
-                    FileUtils.close(br);
+                    });
                 } catch (final IOException e) {
                     if (getFailonerror()) {
                         throw new BuildException(e);
-                    } else {
-                        log("Caught exception: " + e.getMessage(),
-                            Project.MSG_ERR);
                     }
+                    log("Caught exception: " + e.getMessage(), Project.MSG_ERR);
                 }
             }
         } catch (final JSchException e) {
             if (getFailonerror()) {
                 throw new BuildException(e);
-            } else {
-                log("Caught exception: " + e.getMessage(), Project.MSG_ERR);
             }
+            log("Caught exception: " + e.getMessage(), Project.MSG_ERR);
         } finally {
             if (outputProperty != null) {
                 getProject().setNewProperty(outputProperty, output.toString());
@@ -350,7 +347,7 @@ public class SSHExec extends SSHBase {
         }
     }
 
-    private void executeCommand(final Session session, final String cmd, final StringBuffer sb)
+    private void executeCommand(final Session session, final String cmd, final StringBuilder sb)
         throws BuildException {
         final ByteArrayOutputStream out = new ByteArrayOutputStream();
         final ByteArrayOutputStream errout = new ByteArrayOutputStream();
@@ -360,7 +357,7 @@ public class SSHExec extends SSHBase {
         InputStream istream = null;
         if (inputFile != null) {
             try {
-                istream = new FileInputStream(inputFile);
+                istream = Files.newInputStream(inputFile.toPath());
             } catch (final IOException e) {
                 // because we checked the existence before, this one
                 // shouldn't happen What if the file exists, but there
@@ -399,21 +396,18 @@ public class SSHExec extends SSHBase {
             channel.connect();
             // wait for it to finish
             thread =
-                new Thread() {
-                    @Override
-                    public void run() {
+                    new Thread(() -> {
                         while (!channel.isClosed()) {
                             if (thread == null) {
                                 return;
                             }
                             try {
-                                sleep(RETRY_INTERVAL);
+                                Thread.sleep(RETRY_INTERVAL);
                             } catch (final Exception e) {
                                 // ignored
                             }
                         }
-                    }
-                };
+                    });
 
             thread.start();
             thread.join(maxwait);
@@ -423,9 +417,8 @@ public class SSHExec extends SSHBase {
                 thread = null;
                 if (getFailonerror()) {
                     throw new BuildException(TIMEOUT_MESSAGE);
-                } else {
-                    log(TIMEOUT_MESSAGE, Project.MSG_ERR);
                 }
+                log(TIMEOUT_MESSAGE, Project.MSG_ERR);
             } else {
                 // stdout to outputFile
                 if (outputFile != null) {
@@ -450,34 +443,29 @@ public class SSHExec extends SSHBase {
                     final String msg = "Remote command failed with exit status " + ec;
                     if (getFailonerror()) {
                         throw new BuildException(msg);
-                    } else {
-                        log(msg, Project.MSG_ERR);
                     }
+                    log(msg, Project.MSG_ERR);
                 }
             }
         } catch (final BuildException e) {
             throw e;
         } catch (final JSchException e) {
-            if (e.getMessage().indexOf("session is down") >= 0) {
+            if (e.getMessage().contains("session is down")) {
                 if (getFailonerror()) {
                     throw new BuildException(TIMEOUT_MESSAGE, e);
-                } else {
-                    log(TIMEOUT_MESSAGE, Project.MSG_ERR);
                 }
+                log(TIMEOUT_MESSAGE, Project.MSG_ERR);
             } else {
                 if (getFailonerror()) {
                     throw new BuildException(e);
-                } else {
-                    log("Caught exception: " + e.getMessage(),
-                        Project.MSG_ERR);
                 }
+                log("Caught exception: " + e.getMessage(), Project.MSG_ERR);
             }
         } catch (final Exception e) {
             if (getFailonerror()) {
                 throw new BuildException(e);
-            } else {
-                log("Caught exception: " + e.getMessage(), Project.MSG_ERR);
             }
+            log("Caught exception: " + e.getMessage(), Project.MSG_ERR);
         } finally {
             sb.append(out.toString());
             FileUtils.close(istream);
@@ -491,16 +479,14 @@ public class SSHExec extends SSHBase {
      * @param from           string to write
      * @param to             file to write to
      * @param append         if true, append to existing file, else overwrite
-     * @exception Exception  most likely an IOException
      */
     private void writeToFile(final String from, final boolean append, final File to)
         throws IOException {
-        try (FileWriter out = new FileWriter(to.getAbsolutePath(), append)) {
+        try (BufferedWriter out = new BufferedWriter(new FileWriter(to.getAbsolutePath(), append))) {
             final StringReader in = new StringReader(from);
             final char[] buffer = new char[BUFFER_SIZE];
-            int bytesRead;
             while (true) {
-                bytesRead = in.read(buffer);
+                int bytesRead = in.read(buffer);
                 if (bytesRead == -1) {
                     break;
                 }
