@@ -19,35 +19,34 @@
 package org.apache.tools.ant.types;
 
 import java.io.File;
-import java.util.StringTokenizer;
-import java.util.Vector;
 import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
-import java.util.LinkedList;
-import java.util.Iterator;
+import java.util.StringTokenizer;
+import java.util.stream.Stream;
 
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.ProjectComponent;
-import org.apache.tools.ant.util.StringUtils;
 import org.apache.tools.ant.taskdefs.condition.Os;
 
 /**
  * Commandline objects help handling command lines specifying processes to
  * execute.
- *
+ * <p>
  * The class can be used to define a command line as nested elements or as a
  * helper to define a command line by an application.
- * <p>
- * <code>
- * &lt;someelement&gt;<br>
- * &nbsp;&nbsp;&lt;acommandline executable="/executable/to/run"&gt;<br>
- * &nbsp;&nbsp;&nbsp;&nbsp;&lt;argument value="argument 1" /&gt;<br>
- * &nbsp;&nbsp;&nbsp;&nbsp;&lt;argument line="argument_1 argument_2 argument_3" /&gt;<br>
- * &nbsp;&nbsp;&nbsp;&nbsp;&lt;argument value="argument 4" /&gt;<br>
- * &nbsp;&nbsp;&lt;/acommandline&gt;<br>
- * &lt;/someelement&gt;<br>
- * </code>
+ * </p>
+ * <pre>
+ * &lt;someelement&gt;
+ *   &lt;acommandline executable="/executable/to/run"&gt;
+ *     &lt;argument value="argument 1"/&gt;
+ *     &lt;argument line="argument_1 argument_2 argument_3"/&gt;
+ *     &lt;argument value="argument 4"/&gt;
+ *   &lt;/acommandline&gt;
+ * &lt;/someelement&gt;
+ * </pre>
  * The element <code>someelement</code> must provide a method
  * <code>createAcommandline</code> which returns an instance of this class.
  *
@@ -59,19 +58,15 @@ public class Commandline implements Cloneable {
     /**
      * The arguments of the command
      */
-    private Vector arguments = new Vector();
+    private List<Argument> arguments = new ArrayList<>();
 
     /**
      * the program to execute
      */
     private String executable = null;
 
-    protected static final String DISCLAIMER =
-        StringUtils.LINE_SEP
-        + "The \' characters around the executable and arguments are"
-        + StringUtils.LINE_SEP
-        + "not part of the command."
-        + StringUtils.LINE_SEP;
+    protected static final String DISCLAIMER = String.format(
+            "%nThe ' characters around the executable and arguments are%nnot part of the command.%n");
 
     /**
      * Create a command line from a string.
@@ -183,13 +178,25 @@ public class Commandline implements Cloneable {
             this.suffix = suffix != null ? suffix : "";
         }
 
+        /**
+         * Copies settings from a different argument.
+         *
+         * @param other the argument to copy setting from
+         *
+         * @since Ant 1.10.6
+         */
+        public void copyFrom(Argument other) {
+            this.parts = other.parts;
+            this.prefix = other.prefix;
+            this.suffix = other.suffix;
+        }
+
         /**
          * Return the constituent parts of this Argument.
          * @return an array of strings.
          */
         public String[] getParts() {
-            if (parts == null || parts.length == 0
-                || (prefix.length() == 0 && suffix.length() == 0)) {
+            if (parts == null || parts.length == 0 || (prefix.isEmpty() && suffix.isEmpty())) {
                 return parts;
             }
             String[] fullParts = new String[parts.length];
@@ -230,11 +237,9 @@ public class Commandline implements Cloneable {
          */
         public int getPosition() {
             if (realPos == -1) {
-                realPos = (executable == null ? 0 : 1);
-                for (int i = 0; i < position; i++) {
-                    Argument arg = (Argument) arguments.elementAt(i);
-                    realPos += arg.getParts().length;
-                }
+                realPos = (executable == null ? 0 : 1)
+                        + (int) arguments.stream().limit(position)
+                        .map(Argument::getParts).flatMap(Stream::of).count();
             }
             return realPos;
         }
@@ -252,6 +257,7 @@ public class Commandline implements Cloneable {
         /**
          * Get the prefix to be placed in front of the inserted argument.
          *
+         * @return String
          * @since Ant 1.8.0
          */
         public String getPrefix() {
@@ -271,6 +277,7 @@ public class Commandline implements Cloneable {
         /**
          * Get the suffix to be placed at the end of the inserted argument.
          *
+         * @return String
          * @since Ant 1.8.0
          */
         public String getSuffix() {
@@ -306,9 +313,9 @@ public class Commandline implements Cloneable {
     public Argument createArgument(boolean insertAtStart) {
         Argument argument = new Argument();
         if (insertAtStart) {
-            arguments.insertElementAt(argument, 0);
+            arguments.add(0, argument);
         } else {
-            arguments.addElement(argument);
+            arguments.add(argument);
         }
         return argument;
     }
@@ -319,11 +326,25 @@ public class Commandline implements Cloneable {
      * @param executable the String executable name.
      */
     public void setExecutable(String executable) {
-        if (executable == null || executable.length() == 0) {
+        setExecutable(executable, true);
+    }
+
+    /**
+     * Set the executable to run.
+     *
+     * @param executable the String executable name.
+     * @param translateFileSeparator if {@code true} all file separators in the string
+     * are converted to the platform specific value.
+     *
+     * @since Ant 1.9.7
+     */
+    public void setExecutable(String executable, boolean translateFileSeparator) {
+        if (executable == null || executable.isEmpty()) {
             return;
         }
-        this.executable = executable.replace('/', File.separatorChar)
-            .replace('\\', File.separatorChar);
+        this.executable = translateFileSeparator
+            ? executable.replace('/', File.separatorChar).replace('\\', File.separatorChar)
+            : executable;
     }
 
     /**
@@ -339,8 +360,8 @@ public class Commandline implements Cloneable {
      * @param line an array of arguments to append.
      */
     public void addArguments(String[] line) {
-        for (int i = 0; i < line.length; i++) {
-            createArgument().setValue(line[i]);
+        for (String l : line) {
+            createArgument().setValue(l);
         }
     }
 
@@ -349,11 +370,9 @@ public class Commandline implements Cloneable {
      * @return the commandline as an array of strings.
      */
     public String[] getCommandline() {
-        List commands = new LinkedList();
-        ListIterator list = commands.listIterator();
-        addCommandToList(list);
-        final String[] result = new String[commands.size()];
-        return (String[]) commands.toArray(result);
+        final List<String> commands = new LinkedList<>();
+        addCommandToList(commands.listIterator());
+        return commands.toArray(new String[commands.size()]);
     }
 
     /**
@@ -361,7 +380,7 @@ public class Commandline implements Cloneable {
      * @param list the list to add to.
      * @since Ant 1.6
      */
-    public void addCommandToList(ListIterator list) {
+    public void addCommandToList(ListIterator<String> list) {
         if (executable != null) {
             list.add(executable);
         }
@@ -374,10 +393,9 @@ public class Commandline implements Cloneable {
      * @return the arguments as an array of strings.
      */
     public String[] getArguments() {
-        List result = new ArrayList(arguments.size() * 2);
+        List<String> result = new ArrayList<>(arguments.size() * 2);
         addArgumentsToList(result.listIterator());
-        String [] res = new String[result.size()];
-        return (String[]) result.toArray(res);
+        return result.toArray(new String[result.size()]);
     }
 
     /**
@@ -385,14 +403,12 @@ public class Commandline implements Cloneable {
      * @param list the list of arguments.
      * @since Ant 1.6
      */
-    public void addArgumentsToList(ListIterator list) {
-        final int size = arguments.size();
-        for (int i = 0; i < size; i++) {
-            Argument arg = (Argument) arguments.elementAt(i);
+    public void addArgumentsToList(ListIterator<String> list) {
+        for (Argument arg : arguments) {
             String[] s = arg.getParts();
             if (s != null) {
-                for (int j = 0; j < s.length; j++) {
-                    list.add(s[j]);
+                for (String value : s) {
+                    list.add(value);
                 }
             }
         }
@@ -402,6 +418,7 @@ public class Commandline implements Cloneable {
      * Return the command line as a string.
      * @return the command line.
      */
+    @Override
     public String toString() {
         return toString(getCommandline());
     }
@@ -418,21 +435,19 @@ public class Commandline implements Cloneable {
      *                           and double quotes.
      */
     public static String quoteArgument(String argument) {
-        if (argument.indexOf("\"") > -1) {
-            if (argument.indexOf("\'") > -1) {
+        if (argument.contains("\"")) {
+            if (argument.contains("\'")) {
                 throw new BuildException("Can\'t handle single and double"
                         + " quotes in same argument");
-            } else {
-                return '\'' + argument + '\'';
             }
-        } else if (argument.indexOf("\'") > -1
-                   || argument.indexOf(" ") > -1
-                   // WIN9x uses a bat file for executing commands
-                   || (IS_WIN_9X && argument.indexOf(';') != -1)) {
-            return '\"' + argument + '\"';
-        } else {
-            return argument;
+            return '\'' + argument + '\'';
         }
+        if (argument.contains("\'") || argument.contains(" ")
+                // WIN9x uses a bat file for executing commands
+                || (IS_WIN_9X && argument.contains(";"))) {
+            return '\"' + argument + '\"';
+        }
+        return argument;
     }
 
     /**
@@ -448,12 +463,12 @@ public class Commandline implements Cloneable {
             return "";
         }
         // path containing one or more elements
-        final StringBuffer result = new StringBuffer();
-        for (int i = 0; i < line.length; i++) {
-            if (i > 0) {
+        final StringBuilder result = new StringBuilder();
+        for (String l : line) {
+            if (result.length() > 0) {
                 result.append(' ');
             }
-            result.append(quoteArgument(line[i]));
+            result.append(quoteArgument(l));
         }
         return result.toString();
     }
@@ -465,7 +480,7 @@ public class Commandline implements Cloneable {
      * An empty or null toProcess parameter results in a zero sized array.
      */
     public static String[] translateCommandline(String toProcess) {
-        if (toProcess == null || toProcess.length() == 0) {
+        if (toProcess == null || toProcess.isEmpty()) {
             //no command? no string
             return new String[0];
         }
@@ -475,9 +490,9 @@ public class Commandline implements Cloneable {
         final int inQuote = 1;
         final int inDoubleQuote = 2;
         int state = normal;
-        StringTokenizer tok = new StringTokenizer(toProcess, "\"\' ", true);
-        Vector v = new Vector();
-        StringBuffer current = new StringBuffer();
+        final StringTokenizer tok = new StringTokenizer(toProcess, "\"\' ", true);
+        final ArrayList<String> result = new ArrayList<>();
+        final StringBuilder current = new StringBuilder();
         boolean lastTokenHasBeenQuoted = false;
 
         while (tok.hasMoreTokens()) {
@@ -505,9 +520,9 @@ public class Commandline implements Cloneable {
                 } else if ("\"".equals(nextTok)) {
                     state = inDoubleQuote;
                 } else if (" ".equals(nextTok)) {
-                    if (lastTokenHasBeenQuoted || current.length() != 0) {
-                        v.addElement(current.toString());
-                        current = new StringBuffer();
+                    if (lastTokenHasBeenQuoted || current.length() > 0) {
+                        result.add(current.toString());
+                        current.setLength(0);
                     }
                 } else {
                     current.append(nextTok);
@@ -516,15 +531,13 @@ public class Commandline implements Cloneable {
                 break;
             }
         }
-        if (lastTokenHasBeenQuoted || current.length() != 0) {
-            v.addElement(current.toString());
+        if (lastTokenHasBeenQuoted || current.length() > 0) {
+            result.add(current.toString());
         }
         if (state == inQuote || state == inDoubleQuote) {
             throw new BuildException("unbalanced quotes in " + toProcess);
         }
-        String[] args = new String[v.size()];
-        v.copyInto(args);
-        return args;
+        return result.toArray(new String[result.size()]);
     }
 
     /**
@@ -540,10 +553,11 @@ public class Commandline implements Cloneable {
      * Generate a deep clone of the contained object.
      * @return a clone of the contained object
      */
+    @Override
     public Object clone() {
         try {
             Commandline c = (Commandline) super.clone();
-            c.arguments = (Vector) arguments.clone();
+            c.arguments = new ArrayList<>(arguments);
             return c;
         } catch (CloneNotSupportedException e) {
             throw new BuildException(e);
@@ -555,7 +569,7 @@ public class Commandline implements Cloneable {
      */
     public void clear() {
         executable = null;
-        arguments.removeAllElements();
+        arguments.clear();
     }
 
     /**
@@ -563,7 +577,7 @@ public class Commandline implements Cloneable {
      * another operation.
      */
     public void clearArgs() {
-        arguments.removeAllElements();
+        arguments.clear();
     }
 
     /**
@@ -580,7 +594,7 @@ public class Commandline implements Cloneable {
 
     /**
      * Return a String that describes the command and arguments suitable for
-     * verbose output before a call to <code>Runtime.exec(String[])<code>.
+     * verbose output before a call to <code>Runtime.exec(String[])</code>.
      * @return a string that describes the command and arguments.
      * @since Ant 1.5
      */
@@ -590,7 +604,7 @@ public class Commandline implements Cloneable {
 
     /**
      * Return a String that describes the arguments suitable for
-     * verbose output before a call to <code>Runtime.exec(String[])<code>.
+     * verbose output before a call to <code>Runtime.exec(String[])</code>.
      * @return a string that describes the arguments.
      * @since Ant 1.5
      */
@@ -600,7 +614,7 @@ public class Commandline implements Cloneable {
 
     /**
      * Return a String that describes the command and arguments suitable for
-     * verbose output before a call to <code>Runtime.exec(String[])<code>.
+     * verbose output before a call to <code>Runtime.exec(String[])</code>.
      * @param line the Commandline to describe.
      * @return a string that describes the command and arguments.
      * @since Ant 1.5
@@ -611,7 +625,7 @@ public class Commandline implements Cloneable {
 
     /**
      * Return a String that describes the arguments suitable for
-     * verbose output before a call to <code>Runtime.exec(String[])<code>.
+     * verbose output before a call to <code>Runtime.exec(String[])</code>.
      * @param line the Commandline whose arguments to describe.
      * @return a string that describes the arguments.
      * @since Ant 1.5
@@ -622,7 +636,7 @@ public class Commandline implements Cloneable {
 
     /**
      * Return a String that describes the command and arguments suitable for
-     * verbose output before a call to <code>Runtime.exec(String[])<code>.
+     * verbose output before a call to <code>Runtime.exec(String[])</code>.
      *
      * <p>This method assumes that the first entry in the array is the
      * executable to run.</p>
@@ -634,9 +648,7 @@ public class Commandline implements Cloneable {
         if (args == null || args.length == 0) {
             return "";
         }
-        StringBuffer buf = new StringBuffer("Executing \'");
-        buf.append(args[0]);
-        buf.append("\'");
+        StringBuilder buf = new StringBuilder("Executing \'").append(args[0]).append("\'");
         if (args.length > 1) {
             buf.append(" with ");
             buf.append(describeArguments(args, 1));
@@ -648,7 +660,7 @@ public class Commandline implements Cloneable {
 
     /**
      * Return a String that describes the arguments suitable for
-     * verbose output before a call to <code>Runtime.exec(String[])<code>.
+     * verbose output before a call to <code>Runtime.exec(String[])</code>.
      * @param args the command line to describe as an array of strings.
      * @return a string that describes the arguments.
      * @since Ant 1.5
@@ -659,7 +671,7 @@ public class Commandline implements Cloneable {
 
     /**
      * Return a String that describes the arguments suitable for
-     * verbose output before a call to <code>Runtime.exec(String[])<code>.
+     * verbose output before a call to <code>Runtime.exec(String[])</code>.
      *
      * @param args the command line to describe as an array of strings.
      * @param offset ignore entries before this index.
@@ -671,14 +683,10 @@ public class Commandline implements Cloneable {
         if (args == null || args.length <= offset) {
             return "";
         }
-        StringBuffer buf = new StringBuffer("argument");
-        if (args.length > offset) {
-            buf.append("s");
-        }
-        buf.append(":").append(StringUtils.LINE_SEP);
+        StringBuilder buf = new StringBuilder();
+        buf.append(String.format("argument%s:%n", args.length > offset ? "s" : ""));
         for (int i = offset; i < args.length; i++) {
-            buf.append("\'").append(args[i]).append("\'")
-                .append(StringUtils.LINE_SEP);
+            buf.append(String.format("\'%s\'%n", args[i]));
         }
         buf.append(DISCLAIMER);
         return buf.toString();
@@ -689,7 +697,7 @@ public class Commandline implements Cloneable {
      * @since Ant 1.7
      * @return an Iterator.
      */
-    public Iterator iterator() {
+    public Iterator<Argument> iterator() {
         return arguments.iterator();
     }
 }
