@@ -18,7 +18,6 @@
 
 package org.apache.tools.ant.types;
 
-import java.util.Enumeration;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
@@ -31,7 +30,7 @@ import org.apache.tools.ant.util.JavaEnvUtils;
 
 /**
  * A representation of a Java command line that is
- * a composite of 2 <tt>Commandline</tt>s. One is used for the
+ * a composite of 2 <code>Commandline</code>s. One is used for the
  * vm/options and one for the classname/arguments. It provides
  * specific methods for a Java command line.
  *
@@ -82,7 +81,7 @@ public class CommandlineJava implements Cloneable {
         /** the system properties. */
         Properties sys = null;
         // CheckStyle:VisibilityModifier ON
-        private Vector<PropertySet> propertySets = new Vector<PropertySet>();
+        private Vector<PropertySet> propertySets = new Vector<>();
 
         /**
          * Get the properties as an array; this is an override of the
@@ -90,15 +89,15 @@ public class CommandlineJava implements Cloneable {
          * @return the array of definitions; may be null.
          * @throws BuildException on error.
          */
+        @Override
         public String[] getVariables() throws BuildException {
 
-            List<String> definitions = new LinkedList<String>();
+            List<String> definitions = new LinkedList<>();
             addDefinitionsToList(definitions.listIterator());
-            if (definitions.size() == 0) {
+            if (definitions.isEmpty()) {
                 return null;
-            } else {
-                return definitions.toArray(new String[definitions.size()]);
             }
+            return definitions.toArray(new String[definitions.size()]);
         }
 
         /**
@@ -108,16 +107,13 @@ public class CommandlineJava implements Cloneable {
         public void addDefinitionsToList(ListIterator<String> listIt) {
             String[] props = super.getVariables();
             if (props != null) {
-                for (int i = 0; i < props.length; i++) {
-                    listIt.add("-D" + props[i]);
+                for (String prop : props) {
+                    listIt.add("-D" + prop);
                 }
             }
             Properties propertySetProperties = mergePropertySets();
-            for (Enumeration<?> e = propertySetProperties.keys();
-                 e.hasMoreElements();) {
-                String key = (String) e.nextElement();
-                String value = propertySetProperties.getProperty(key);
-                listIt.add("-D" + key + "=" + value);
+            for (String key : propertySetProperties.stringPropertyNames()) {
+                listIt.add("-D" + key + "=" + propertySetProperties.getProperty(key));
             }
         }
 
@@ -140,10 +136,9 @@ public class CommandlineJava implements Cloneable {
             try {
                 sys = System.getProperties();
                 Properties p = new Properties();
-                for (Enumeration<?> e = sys.propertyNames(); e.hasMoreElements();) {
-                    String name = (String) e.nextElement();
+                for (String name : sys.stringPropertyNames()) {
                     String value = sys.getProperty(name);
-                    if (name != null && value != null) {
+                    if (value != null) {
                         p.put(name, value);
                     }
                 }
@@ -182,6 +177,7 @@ public class CommandlineJava implements Cloneable {
          * @exception CloneNotSupportedException for signature.
          */
         @SuppressWarnings("unchecked")
+        @Override
         public Object clone() throws CloneNotSupportedException {
             try {
                 SysProperties c = (SysProperties) super.clone();
@@ -329,7 +325,7 @@ public class CommandlineJava implements Cloneable {
     /**
      * Get the name of the jar to be run.
      * @return the pathname of the jar file to run via -jar option
-     * or <tt>null</tt> if there is no jar to run.
+     * or <code>null</code> if there is no jar to run.
      * @see #getClassname()
      */
     public String getJar() {
@@ -356,7 +352,7 @@ public class CommandlineJava implements Cloneable {
 
     /**
      * Get the name of the class to be run.
-     * @return the name of the class to run or <tt>null</tt> if there is no class.
+     * @return the name of the class to run or <code>null</code> if there is no class.
      * @see #getJar()
      */
     public String getClassname() {
@@ -366,11 +362,34 @@ public class CommandlineJava implements Cloneable {
                     return javaCommand.getExecutable();
                 case MODULE:
                     return parseClassFromModuleClassPair(javaCommand.getExecutable());
+                default:
             }
         }
         return null;
     }
 
+    /**
+     * Set the source-file, to execute as single file source programs, a feature, available
+     * since Java 11.
+     *
+     * @param sourceFile The path to the source file
+     * @since Ant 1.10.5
+     */
+    public void setSourceFile(final String sourceFile) {
+        this.executableType = ExecutableType.SOURCE_FILE;
+        javaCommand.setExecutable(sourceFile);
+    }
+
+    /**
+     * @return Returns the source-file to execute, if this command line has
+     * been {@link #setSourceFile(String) configured for single file source program
+     * execution}. Else returns null.
+     * @since Ant 1.10.5
+     */
+    public String getSourceFile() {
+        return this.executableType == ExecutableType.SOURCE_FILE ? this.javaCommand.getExecutable() : null;
+    }
+
     /**
      * Set the module to execute.
      * @param module  the module name.
@@ -385,16 +404,14 @@ public class CommandlineJava implements Cloneable {
                     javaCommand.setExecutable(module, false);
                     break;
                 case CLASS:
-                    javaCommand.setExecutable(createModuleClassPair(
-                            module,
+                    javaCommand.setExecutable(createModuleClassPair(module,
                             javaCommand.getExecutable()), false);
                     break;
                 case MODULE:
-                    javaCommand.setExecutable(createModuleClassPair(
-                            module,
-                            parseClassFromModuleClassPair(javaCommand.getExecutable())),
-                                              false);
+                    javaCommand.setExecutable(createModuleClassPair(module,
+                            parseClassFromModuleClassPair(javaCommand.getExecutable())), false);
                     break;
+                default:
             }
         }
         executableType = ExecutableType.MODULE;
@@ -402,7 +419,7 @@ public class CommandlineJava implements Cloneable {
 
     /**
      * Get the name of the module to be run.
-     * @return the name of the module to run or <tt>null</tt> if there is no module.
+     * @return the name of the module to run or <code>null</code> if there is no module.
      * @see #getJar()
      * @see #getClassname()
      * @since 1.9.7
@@ -479,7 +496,7 @@ public class CommandlineJava implements Cloneable {
      */
     public String[] getCommandline() {
         //create the list
-        List<String> commands = new LinkedList<String>();
+        List<String> commands = new LinkedList<>();
         //fill it
         addCommandsToList(commands.listIterator());
         //convert to an array
@@ -500,8 +517,7 @@ public class CommandlineJava implements Cloneable {
         if (isCloneVm()) {
             SysProperties clonedSysProperties = new SysProperties();
             PropertySet ps = new PropertySet();
-            PropertySet.BuiltinPropertySetName sys =
-                new PropertySet.BuiltinPropertySetName();
+            PropertySet.BuiltinPropertySetName sys = new PropertySet.BuiltinPropertySetName();
             sys.setValue("system");
             ps.appendBuiltin(sys);
             clonedSysProperties.addSyspropertyset(ps);
@@ -515,20 +531,17 @@ public class CommandlineJava implements Cloneable {
         //main classpath
         if (haveClasspath()) {
             listIterator.add("-classpath");
-            listIterator.add(
-                    classpath.concatSystemClasspath("ignore").toString());
+            listIterator.add(classpath.concatSystemClasspath("ignore").toString());
         }
         //module path
         if (haveModulepath()) {
             listIterator.add("--module-path");
-            listIterator.add(
-                    modulepath.concatSystemClasspath("ignore").toString());
+            listIterator.add(modulepath.concatSystemClasspath("ignore").toString());
         }
         //upgrade module path
         if (haveUpgrademodulepath()) {
             listIterator.add("--upgrade-module-path");
-            listIterator.add(
-                    upgrademodulepath.concatSystemClasspath("ignore").toString());
+            listIterator.add(upgrademodulepath.concatSystemClasspath("ignore").toString());
         }
         //now any assertions are added
         if (getAssertions() != null) {
@@ -543,9 +556,11 @@ public class CommandlineJava implements Cloneable {
         } else if (executableType == ExecutableType.MODULE) {
             listIterator.add("-m");
         }
-        // this is the classname to run as well as its arguments.
+        // this is the classname/source-file to run as well as its arguments.
         // in case of ExecutableType.JAR, the executable is a jar file,
-        // in case of ExecutableType.MODULE, the executable is a module name, portentially including a class name.
+        // in case of ExecutableType.MODULE, the executable is a module name, potentially including a class name.
+        // in case of ExecutableType.SOURCE_FILE, the executable is a Java source file (ending in .java) or a shebang
+        // file containing Java source
         javaCommand.addCommandToList(listIterator);
     }
 
@@ -562,13 +577,14 @@ public class CommandlineJava implements Cloneable {
      * Get a string description.
      * @return the command line as a string.
      */
+    @Override
     public String toString() {
         return Commandline.toString(getCommandline());
     }
 
     /**
      * Return a String that describes the command and arguments suitable for
-     * verbose output before a call to <code>Runtime.exec(String[])<code>.
+     * verbose output before a call to <code>Runtime.exec(String[])</code>.
      * @return the description string.
      * @since Ant 1.5
      */
@@ -610,9 +626,10 @@ public class CommandlineJava implements Cloneable {
      * @return the total number of arguments in the java command line.
      * @see #getCommandline()
      * @deprecated since 1.7.
-     *             Please dont use this, it effectively creates the
+     *             Please don't use this, it effectively creates the
      *             entire command.
      */
+    @Deprecated
     public int size() {
         int size = getActualVMCommand().size() + javaCommand.size()
             + sysProperties.size();
@@ -721,6 +738,7 @@ public class CommandlineJava implements Cloneable {
      * @throws BuildException if anything went wrong.
      * @throws CloneNotSupportedException never.
      */
+    @Override
     public Object clone() throws CloneNotSupportedException {
         try {
             CommandlineJava c = (CommandlineJava) super.clone();
@@ -762,10 +780,8 @@ public class CommandlineJava implements Cloneable {
      * @since Ant 1.6
      */
     public boolean haveClasspath() {
-        Path fullClasspath = classpath != null
-            ? classpath.concatSystemClasspath("ignore") : null;
-        return fullClasspath != null
-            && fullClasspath.toString().trim().length() > 0;
+        Path fullClasspath = classpath == null ? null : classpath.concatSystemClasspath("ignore");
+        return fullClasspath != null && !fullClasspath.toString().trim().isEmpty();
     }
 
     /**
@@ -789,9 +805,9 @@ public class CommandlineJava implements Cloneable {
      */
     public boolean haveModulepath() {
         Path fullClasspath = modulepath != null
-            ? modulepath.concatSystemClasspath("ignore") : null;
+                ? modulepath.concatSystemClasspath("ignore") : null;
         return fullClasspath != null
-            && fullClasspath.toString().trim().length() > 0;
+            && !fullClasspath.toString().trim().isEmpty();
     }
 
     /**
@@ -801,9 +817,8 @@ public class CommandlineJava implements Cloneable {
      */
     public boolean haveUpgrademodulepath() {
         Path fullClasspath = upgrademodulepath != null
-            ? upgrademodulepath.concatSystemClasspath("ignore") : null;
-        return fullClasspath != null
-            && fullClasspath.toString().trim().length() > 0;
+                ? upgrademodulepath.concatSystemClasspath("ignore") : null;
+        return fullClasspath != null && !fullClasspath.toString().trim().isEmpty();
     }
 
     /**
@@ -816,8 +831,7 @@ public class CommandlineJava implements Cloneable {
     private Path calculateBootclasspath(boolean log) {
         if (vmVersion.startsWith("1.1")) {
             if (bootclasspath != null && log) {
-                bootclasspath.log("Ignoring bootclasspath as "
-                                  + "the target VM doesn't support it.");
+                bootclasspath.log("Ignoring bootclasspath as the target VM doesn't support it.");
             }
         } else {
             Path b = bootclasspath;
@@ -839,8 +853,7 @@ public class CommandlineJava implements Cloneable {
      * @since 1.7
      */
     private boolean isCloneVm() {
-        return cloneVm
-            || "true".equals(System.getProperty("ant.build.clonevm"));
+        return cloneVm || Boolean.parseBoolean(System.getProperty("ant.build.clonevm"));
     }
 
     /**
@@ -851,9 +864,7 @@ public class CommandlineJava implements Cloneable {
      * @since 1.9.7
      */
     private static String createModuleClassPair(final String module, final String classname) {
-        return classname == null ?
-                module :
-                String.format("%s/%s", module, classname);   //NOI18N
+        return classname == null ? module : String.format("%s/%s", module, classname);   //NOI18N
     }
 
     /**
@@ -881,9 +892,7 @@ public class CommandlineJava implements Cloneable {
             return null;
         }
         final String[] moduleAndClass = moduleClassPair.split("/");  //NOI18N
-        return moduleAndClass.length == 2 ?
-                moduleAndClass[1] :
-                null;
+        return moduleAndClass.length == 2 ? moduleAndClass[1] : null;
     }
 
     /**
@@ -902,6 +911,11 @@ public class CommandlineJava implements Cloneable {
         /**
          * Module execution.
          */
-        MODULE
+        MODULE,
+
+        /**
+         * Source file (introduced in Java 11)
+         */
+        SOURCE_FILE,
     }
 }
