@@ -20,17 +20,20 @@ package org.apache.tools.ant.types;
 
 import java.io.File;
 import java.lang.reflect.Method;
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Locale;
 import java.util.Stack;
-import java.util.Vector;
 
 import org.apache.tools.ant.BuildException;
+import org.apache.tools.ant.MagicNames;
 import org.apache.tools.ant.PathTokenizer;
 import org.apache.tools.ant.Project;
-import org.apache.tools.ant.types.resources.Union;
 import org.apache.tools.ant.types.resources.FileResourceIterator;
+import org.apache.tools.ant.types.resources.Union;
 import org.apache.tools.ant.util.FileUtils;
 import org.apache.tools.ant.util.JavaEnvUtils;
 
@@ -38,49 +41,45 @@ import org.apache.tools.ant.util.JavaEnvUtils;
  * This object represents a path as used by CLASSPATH or PATH
  * environment variable. A path might also be described as a collection
  * of unique filesystem resources.
+ * <pre>
+ * &lt;sometask&gt;
+ *   &lt;somepath&gt;
+ *     &lt;pathelement location="/path/to/file.jar"/&gt;
+ *     &lt;pathelement path="/path/to/file2.jar:/path/to/class2;/path/to/class3"/&gt;
+ *     &lt;pathelement location="/path/to/file3.jar"/&gt;
+ *     &lt;pathelement location="/path/to/file4.jar"/&gt;
+ *   &lt;/somepath&gt;
+ * &lt;/sometask&gt;
+ * </pre>
  * <p>
- * <code>
- * &lt;sometask&gt;<br>
- * &nbsp;&nbsp;&lt;somepath&gt;<br>
- * &nbsp;&nbsp;&nbsp;&nbsp;&lt;pathelement location="/path/to/file.jar" /&gt;<br>
- * &nbsp;&nbsp;&nbsp;&nbsp;&lt;pathelement
- *  path="/path/to/file2.jar:/path/to/class2;/path/to/class3" /&gt;
- * <br>
- * &nbsp;&nbsp;&nbsp;&nbsp;&lt;pathelement location="/path/to/file3.jar" /&gt;<br>
- * &nbsp;&nbsp;&nbsp;&nbsp;&lt;pathelement location="/path/to/file4.jar" /&gt;<br>
- * &nbsp;&nbsp;&lt;/somepath&gt;<br>
- * &lt;/sometask&gt;<br>
- * </code>
- * <p>
- * The object implemention <code>sometask</code> must provide a method called
+ * The object implementation <code>sometask</code> must provide a method called
  * <code>createSomepath</code> which returns an instance of <code>Path</code>.
  * Nested path definitions are handled by the Path object and must be labeled
- * <code>pathelement</code>.<p>
- *
+ * <code>pathelement</code>.
+ * </p>
+ * <p>
  * The path element takes a parameter <code>path</code> which will be parsed
  * and split into single elements. It will usually be used
  * to define a path from an environment variable.
+ * </p>
  */
 
 public class Path extends DataType implements Cloneable, ResourceCollection {
     // CheckStyle:VisibilityModifier OFF - bc
 
+    // non-final as some IDE integrations (at least Eclipse) want to override it
     /** The system classpath as a Path object */
-    public static Path systemClasspath =
+    public static Path systemClasspath = //NOSONAR
         new Path(null, System.getProperty("java.class.path"));
 
-
     /**
      * The system bootclasspath as a Path object.
      *
      * @since Ant 1.6.2
      */
-    public static Path systemBootClasspath =
+    public static final Path systemBootClasspath =
         new Path(null, System.getProperty("sun.boot.class.path"));
 
-    private static final Iterator EMPTY_ITERATOR
-        = Collections.EMPTY_SET.iterator();
-
     // CheckStyle:VisibilityModifier OFF - bc
 
     /**
@@ -120,7 +119,8 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
          * Create an iterator.
          * @return an iterator.
          */
-        public Iterator iterator() {
+        @Override
+        public Iterator<Resource> iterator() {
             return new FileResourceIterator(getProject(), null, parts);
         }
 
@@ -128,6 +128,7 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
          * Check if this resource is only for filesystems.
          * @return true.
          */
+        @Override
         public boolean isFilesystemOnly() {
             return true;
         }
@@ -136,6 +137,7 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
          * Get the number of resources.
          * @return the number of parts.
          */
+        @Override
         public int size() {
             return parts == null ? 0 : parts.length;
         }
@@ -195,6 +197,7 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      * @param r the reference to another Path
      * @throws BuildException on error
      */
+    @Override
     public void setRefid(Reference r) throws BuildException {
         if (union != null) {
             throw tooManyAttributes();
@@ -328,20 +331,25 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      * @param tryUserDir  if true try the user directory if the file is not present
      */
     public void addExisting(Path source, boolean tryUserDir) {
-        String[] list = source.list();
         File userDir = (tryUserDir) ? new File(System.getProperty("user.dir"))
                 : null;
 
-        for (int i = 0; i < list.length; i++) {
-            File f = resolveFile(getProject(), list[i]);
+        for (String name : source.list()) {
+            File f = resolveFile(getProject(), name);
 
             // probably not the best choice, but it solves the problem of
             // relative paths in CLASSPATH
             if (tryUserDir && !f.exists()) {
-                f = new File(userDir, list[i]);
+                f = new File(userDir, name);
             }
             if (f.exists()) {
                 setLocation(f);
+            } else if (f.getParentFile() != null && f.getParentFile().exists()
+                       && containsWildcards(f.getName())) {
+                setLocation(f);
+                log("adding " + f
+                    + " which contains wildcards and may not do what you intend it to do depending on your OS or version of Java",
+                    Project.MSG_VERBOSE);
             } else {
                 log("dropping " + f + " from path as it doesn't exist",
                     Project.MSG_VERBOSE);
@@ -351,6 +359,7 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
 
     /**
      * Whether to cache the current path.
+     * @param b boolean
      * @since Ant 1.8.0
      */
     public void setCache(boolean b) {
@@ -367,7 +376,7 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      */
     public String[] list() {
         if (isReference()) {
-            return ((Path) getCheckedRef()).list();
+            return getRef().list();
         }
         return assertFilesystemOnly(union) == null
             ? new String[0] : union.list();
@@ -378,8 +387,9 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      * CLASSPATH or PATH environment variable definition.
      * @return a textual representation of the path.
      */
+    @Override
     public String toString() {
-        return isReference() ? getCheckedRef().toString()
+        return isReference() ? getRef().toString()
             : union == null ? "" : union.toString();
     }
 
@@ -390,13 +400,13 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      * @return an array of strings, one for each path element
      */
     public static String[] translatePath(Project project, String source) {
-        final Vector result = new Vector();
         if (source == null) {
             return new String[0];
         }
+        final List<String> result = new ArrayList<>();
         PathTokenizer tok = new PathTokenizer(source);
-        StringBuffer element = new StringBuffer();
         while (tok.hasMoreTokens()) {
+            StringBuffer element = new StringBuffer();
             String pathElement = tok.nextToken();
             try {
                 element.append(resolveFile(project, pathElement).getPath());
@@ -408,12 +418,9 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
             for (int i = 0; i < element.length(); i++) {
                 translateFileSep(element, i);
             }
-            result.addElement(element.toString());
-            element = new StringBuffer();
+            result.add(element.toString());
         }
-        String[] res = new String[result.size()];
-        result.copyInto(res);
-        return res;
+        return result.toArray(new String[result.size()]);
     }
 
     /**
@@ -453,9 +460,10 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      * Fulfill the ResourceCollection contract.
      * @return number of elements as int.
      */
+    @Override
     public synchronized int size() {
         if (isReference()) {
-            return ((Path) getCheckedRef()).size();
+            return getRef().size();
         }
         dieOnCircularReference();
         return union == null ? 0 : assertFilesystemOnly(union).size();
@@ -465,6 +473,7 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      * Clone this Path.
      * @return Path with shallowly cloned Resource children.
      */
+    @Override
     public Object clone() {
         try {
             Path result = (Path) super.clone();
@@ -482,7 +491,8 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      * @param p   the project to use to dereference the references.
      * @throws BuildException on error.
      */
-    protected synchronized void dieOnCircularReference(Stack stk, Project p)
+    @Override
+    protected synchronized void dieOnCircularReference(Stack<Object> stk, Project p)
         throws BuildException {
         if (isChecked()) {
             return;
@@ -546,29 +556,30 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
         Path result = new Path(getProject());
 
         String order = defValue;
-        if (getProject() != null) {
-            String o = getProject().getProperty("build.sysclasspath");
-            if (o != null) {
-                order = o;
-            }
+        String o = getProject() != null
+            ? getProject().getProperty(MagicNames.BUILD_SYSCLASSPATH)
+            : System.getProperty(MagicNames.BUILD_SYSCLASSPATH);
+        if (o != null) {
+            order = o;
         }
-        if (order.equals("only")) {
+        if ("only".equals(order)) {
             // only: the developer knows what (s)he is doing
             result.addExisting(p, true);
 
-        } else if (order.equals("first")) {
+        } else if ("first".equals(order)) {
             // first: developer could use a little help
             result.addExisting(p, true);
             result.addExisting(this);
 
-        } else if (order.equals("ignore")) {
+        } else if ("ignore".equals(order)) {
             // ignore: don't trust anyone
             result.addExisting(this);
 
         } else {
             // last: don't trust the developer
-            if (!order.equals("last")) {
-                log("invalid value for build.sysclasspath: " + order,
+            if (!"last".equals(order)) {
+                log("invalid value for " + MagicNames.BUILD_SYSCLASSPATH
+                    + ": " + order,
                     Project.MSG_WARN);
             }
             result.addExisting(this);
@@ -584,7 +595,7 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
         if (JavaEnvUtils.isKaffe()) {
             // newer versions of Kaffe (1.1.1+) won't have this,
             // but this will be sorted by FileSet anyway.
-            File kaffeShare = new File(System.getProperty("java.home")
+            File kaffeShare = new File(JavaEnvUtils.getJavaHome()
                                        + File.separator + "share"
                                        + File.separator + "kaffe");
             if (kaffeShare.isDirectory()) {
@@ -597,69 +608,53 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
             addExisting(systemBootClasspath);
         }
 
-        if (System.getProperty("java.vendor").toLowerCase(Locale.US).indexOf("microsoft") >= 0) {
-            // XXX is this code still necessary? is there any 1.2+ port?
+        if (System.getProperty("java.vendor").toLowerCase(Locale.ENGLISH).contains("microsoft")) {
+            // TODO is this code still necessary? is there any 1.2+ port?
             // Pull in *.zip from packages directory
             FileSet msZipFiles = new FileSet();
-            msZipFiles.setDir(new File(System.getProperty("java.home")
-                + File.separator + "Packages"));
+            msZipFiles.setDir(new File(JavaEnvUtils.getJavaHome()
+                    + File.separator + "Packages"));
             msZipFiles.setIncludes("*.ZIP");
             addFileset(msZipFiles);
         } else {
             // JDK 1.2+ seems to set java.home to the JRE directory.
-            addExisting(new Path(null,
-                                 System.getProperty("java.home")
-                                 + File.separator + "lib"
-                                 + File.separator + "rt.jar"));
+            addExisting(new Path(null, JavaEnvUtils.getJavaHome()
+                    + File.separator + "lib" + File.separator + "rt.jar"));
             // Just keep the old version as well and let addExisting
             // sort it out.
-            addExisting(new Path(null,
-                                 System.getProperty("java.home")
-                                 + File.separator + "jre"
-                                 + File.separator + "lib"
-                                 + File.separator + "rt.jar"));
+            addExisting(new Path(null, JavaEnvUtils.getJavaHome()
+                    + File.separator + "jre" + File.separator + "lib"
+                    + File.separator + "rt.jar"));
 
             // Sun's and Apple's 1.4 have JCE and JSSE in separate jars.
-            String[] secJars = {"jce", "jsse"};
-            for (int i = 0; i < secJars.length; i++) {
-                addExisting(new Path(null,
-                                     System.getProperty("java.home")
-                                     + File.separator + "lib"
-                                     + File.separator + secJars[i] + ".jar"));
-                addExisting(new Path(null,
-                                     System.getProperty("java.home")
-                                     + File.separator + ".."
-                                     + File.separator + "Classes"
-                                     + File.separator + secJars[i] + ".jar"));
+            for (String secJar : Arrays.asList("jce", "jsse")) {
+                addExisting(new Path(null, JavaEnvUtils.getJavaHome()
+                        + File.separator + "lib"
+                        + File.separator + secJar + ".jar"));
+                addExisting(new Path(null, JavaEnvUtils.getJavaHome()
+                        + File.separator + ".." + File.separator + "Classes"
+                        + File.separator + secJar + ".jar"));
             }
 
             // IBM's 1.4 has rt.jar split into 4 smaller jars and a combined
             // JCE/JSSE in security.jar.
-            String[] ibmJars
-                = {"core", "graphics", "security", "server", "xml"};
-            for (int i = 0; i < ibmJars.length; i++) {
-                addExisting(new Path(null,
-                                     System.getProperty("java.home")
-                                     + File.separator + "lib"
-                                     + File.separator + ibmJars[i] + ".jar"));
+            for (String ibmJar : Arrays.asList("core", "graphics", "security", "server", "xml")) {
+                addExisting(new Path(null, JavaEnvUtils.getJavaHome()
+                        + File.separator + "lib" + File.separator + ibmJar + ".jar"));
             }
 
             // Added for MacOS X
-            addExisting(new Path(null,
-                                 System.getProperty("java.home")
-                                 + File.separator + ".."
-                                 + File.separator + "Classes"
-                                 + File.separator + "classes.jar"));
-            addExisting(new Path(null,
-                                 System.getProperty("java.home")
-                                 + File.separator + ".."
-                                 + File.separator + "Classes"
-                                 + File.separator + "ui.jar"));
+            addExisting(new Path(null, JavaEnvUtils.getJavaHome()
+                    + File.separator + ".." + File.separator + "Classes"
+                    + File.separator + "classes.jar"));
+            addExisting(new Path(null, JavaEnvUtils.getJavaHome()
+                    + File.separator + ".." + File.separator + "Classes"
+                    + File.separator + "ui.jar"));
         }
     }
 
     /**
-     * Emulation of extdirs feature in java >= 1.2.
+     * Emulation of extdirs feature in Java &gt;= 1.2.
      * This method adds all files in the given
      * directories (but not in sub-directories!) to the classpath,
      * so that you don't have to specify them all one by one.
@@ -675,9 +670,8 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
             }
         }
 
-        String[] dirs = extdirs.list();
-        for (int i = 0; i < dirs.length; i++) {
-            File dir = resolveFile(getProject(), dirs[i]);
+        for (String d : extdirs.list()) {
+            File dir = resolveFile(getProject(), d);
             if (dir.exists() && dir.isDirectory()) {
                 FileSet fs = new FileSet();
                 fs.setDir(dir);
@@ -693,15 +687,16 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      * are added to this container while the Iterator is in use.
      * @return a "fail-fast" Iterator.
      */
-    public final synchronized Iterator iterator() {
+    @Override
+    public final synchronized Iterator<Resource> iterator() {
         if (isReference()) {
-            return ((Path) getCheckedRef()).iterator();
+            return getRef().iterator();
         }
         dieOnCircularReference();
         if (getPreserveBC()) {
             return new FileResourceIterator(getProject(), null, list());
         }
-        return union == null ? EMPTY_ITERATOR
+        return union == null ? Collections.<Resource> emptySet().iterator()
             : assertFilesystemOnly(union).iterator();
     }
 
@@ -709,9 +704,10 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      * Fulfill the ResourceCollection contract.
      * @return whether this is a filesystem-only resource collection.
      */
+    @Override
     public synchronized boolean isFilesystemOnly() {
         if (isReference()) {
-            return ((Path) getCheckedRef()).isFilesystemOnly();
+            return getRef().isFilesystemOnly();
         }
         dieOnCircularReference();
         assertFilesystemOnly(union);
@@ -726,8 +722,8 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
      */
     protected ResourceCollection assertFilesystemOnly(ResourceCollection rc) {
         if (rc != null && !(rc.isFilesystemOnly())) {
-            throw new BuildException(getDataTypeName()
-                + " allows only filesystem resources.");
+            throw new BuildException("%s allows only filesystem resources.",
+                getDataTypeName());
         }
         return rc;
     }
@@ -745,7 +741,7 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
             return false;
         }
         try {
-            Method listMethod = getClass().getMethod("list", (Class[]) null);
+            Method listMethod = getClass().getMethod("list");
             return !listMethod.getDeclaringClass().equals(Path.class);
         } catch (Exception e) {
             //shouldn't happen, but
@@ -757,6 +753,19 @@ public class Path extends DataType implements Cloneable, ResourceCollection {
         if (preserveBC == null) {
             preserveBC = delegateIteratorToList() ? Boolean.TRUE : Boolean.FALSE;
         }
-        return preserveBC.booleanValue();
+        return preserveBC;
     }
+
+    /**
+     * Does the given file name contain wildcards?
+     * @since Ant 1.8.2
+     */
+    private static boolean containsWildcards(String path) {
+        return path != null && (path.contains("*") || path.contains("?"));
+    }
+
+    private Path getRef() {
+        return getCheckedRef(Path.class);
+    }
+
 }
