@@ -29,13 +29,14 @@ import org.apache.tools.ant.taskdefs.condition.Os;
  * A set of helper methods related to locating executables or checking
  * conditions of a given Java installation.
  *
+ * <p>Starting with Java 10 we've stopped adding <code>JAVA_</code>
+ * and <code>VERSION_</code> attributes for new major version numbers
+ * of the JVM.</p>
+ *
  * @since Ant 1.5
  */
 public final class JavaEnvUtils {
 
-    private JavaEnvUtils() {
-    }
-
     /** Are we on a DOS-based system */
     private static final boolean IS_DOS = Os.isFamily("dos");
     /** Are we on Novell NetWare */
@@ -107,11 +108,13 @@ public final class JavaEnvUtils {
      * Version constant for Java 1.9
      * @deprecated use #JAVA_9 instead
      */
+    @Deprecated
     public static final String JAVA_1_9 = "1.9";
     /**
      * Number Version constant for Java 1.9
      * @deprecated use #VERSION_9 instead
      */
+    @Deprecated
     public static final int VERSION_1_9 = 19;
 
     /**
@@ -128,7 +131,7 @@ public final class JavaEnvUtils {
     /** Whether this is the Kaffe VM */
     private static boolean kaffeDetected;
 
-    /** Wheter this is a GNU Classpath based VM */
+    /** Whether this is a GNU Classpath based VM */
     private static boolean classpathDetected;
 
     /** Whether this is the GNU VM (gcj/gij) */
@@ -140,6 +143,8 @@ public final class JavaEnvUtils {
     /** array of packages in the runtime */
     private static Vector<String> jrePackages;
 
+    private JavaEnvUtils() {
+    }
 
     static {
 
@@ -181,6 +186,14 @@ public final class JavaEnvUtils {
             Class.forName("java.lang.module.ModuleDescriptor");
             javaVersion = JAVA_9;
             javaVersionNumber = VERSION_9;
+            // at least Java9 and this should properly support the purely numeric version property
+            String v = System.getProperty("java.specification.version");
+            DeweyDecimal pv = new DeweyDecimal(v);
+            javaVersionNumber = pv.get(0) * 10;
+            if (pv.getSize() > 1) {
+                javaVersionNumber += pv.get(1);
+            }
+            javaVersion = pv.toString();
         } catch (Throwable t) {
             // swallow as we've hit the max class version that
             // we have
@@ -238,6 +251,7 @@ public final class JavaEnvUtils {
      * be 0 for all versions of Java starting with Java 9.
      * @deprecated use #getParsedJavaVersion instead
      */
+    @Deprecated
     public static int getJavaVersionNumber() {
         return javaVersionNumber;
     }
@@ -448,52 +462,46 @@ public final class JavaEnvUtils {
      */
 
     private static void buildJrePackages() {
-        jrePackages = new Vector<String>();
-        switch(javaVersionNumber) {
-            case VERSION_9:
-            case VERSION_1_8:
-            case VERSION_1_7:
-                jrePackages.addElement("jdk");
-                // fall through
-            case VERSION_1_6:
-            case VERSION_1_5:
-                //In Java1.5, the apache stuff moved.
-                jrePackages.addElement("com.sun.org.apache");
-                //fall through.
-            case VERSION_1_4:
-                if (javaVersionNumber == VERSION_1_4) {
-                    jrePackages.addElement("org.apache.crimson");
-                    jrePackages.addElement("org.apache.xalan");
-                    jrePackages.addElement("org.apache.xml");
-                    jrePackages.addElement("org.apache.xpath");
-                }
-                jrePackages.addElement("org.ietf.jgss");
-                jrePackages.addElement("org.w3c.dom");
-                jrePackages.addElement("org.xml.sax");
-                // fall through
-            case VERSION_1_3:
-                jrePackages.addElement("org.omg");
-                jrePackages.addElement("com.sun.corba");
-                jrePackages.addElement("com.sun.jndi");
-                jrePackages.addElement("com.sun.media");
-                jrePackages.addElement("com.sun.naming");
-                jrePackages.addElement("com.sun.org.omg");
-                jrePackages.addElement("com.sun.rmi");
-                jrePackages.addElement("sunw.io");
-                jrePackages.addElement("sunw.util");
-                // fall through
-            case VERSION_1_2:
-                jrePackages.addElement("com.sun.java");
-                jrePackages.addElement("com.sun.image");
-                // are there any here that we forgot?
-                // fall through
-            case VERSION_1_1:
-            default:
-                //things like sun.reflection, sun.misc, sun.net
-                jrePackages.addElement("sun");
-                jrePackages.addElement("java");
-                jrePackages.addElement("javax");
-                break;
+        jrePackages = new Vector<>();
+        if (isAtLeastJavaVersion(JAVA_1_1)) {
+            //things like sun.reflection, sun.misc, sun.net
+            jrePackages.addElement("sun");
+            jrePackages.addElement("java");
+            jrePackages.addElement("javax");
+        }
+        if (isAtLeastJavaVersion(JAVA_1_2)) {
+            jrePackages.addElement("com.sun.java");
+            jrePackages.addElement("com.sun.image");
+            // are there any here that we forgot?
+        }
+        if (isAtLeastJavaVersion(JAVA_1_3)) {
+            jrePackages.addElement("org.omg");
+            jrePackages.addElement("com.sun.corba");
+            jrePackages.addElement("com.sun.jndi");
+            jrePackages.addElement("com.sun.media");
+            jrePackages.addElement("com.sun.naming");
+            jrePackages.addElement("com.sun.org.omg");
+            jrePackages.addElement("com.sun.rmi");
+            jrePackages.addElement("sunw.io");
+            jrePackages.addElement("sunw.util");
+        }
+        if (isAtLeastJavaVersion(JAVA_1_4)) {
+            if (javaVersionNumber == VERSION_1_4) {
+                jrePackages.addElement("org.apache.crimson");
+                jrePackages.addElement("org.apache.xalan");
+                jrePackages.addElement("org.apache.xml");
+                jrePackages.addElement("org.apache.xpath");
+            }
+            jrePackages.addElement("org.ietf.jgss");
+            jrePackages.addElement("org.w3c.dom");
+            jrePackages.addElement("org.xml.sax");
+        }
+        if (isAtLeastJavaVersion(JAVA_1_5)) {
+            //In Java1.5, the apache stuff moved.
+            jrePackages.addElement("com.sun.org.apache");
+        }
+        if (isAtLeastJavaVersion(JAVA_1_7)) {
+            jrePackages.addElement("jdk");
         }
     }
 
@@ -502,54 +510,47 @@ public final class JavaEnvUtils {
      * @return a list of test classes depending on the java version.
      */
     public static Vector<String> getJrePackageTestCases() {
-        Vector<String> tests = new Vector<String>();
+        Vector<String> tests = new Vector<>();
         tests.addElement("java.lang.Object");
-        switch(javaVersionNumber) {
-            case VERSION_9:
-            case VERSION_1_8:
-            case VERSION_1_7:
-                tests.addElement("jdk.net.Sockets");
-                // fall through
-            case VERSION_1_6:
-            case VERSION_1_5:
-                tests.addElement(
-                    "com.sun.org.apache.xerces.internal.jaxp.datatype.DatatypeFactoryImpl ");
-                // Fall through
-            case VERSION_1_4:
-                tests.addElement("sun.audio.AudioPlayer");
-                if (javaVersionNumber == VERSION_1_4) {
-                	// only for 1.4, not for higher versions which fall through
-                    tests.addElement("org.apache.crimson.parser.ContentModel");
-                    tests.addElement("org.apache.xalan.processor.ProcessorImport");
-                    tests.addElement("org.apache.xml.utils.URI");
-                    tests.addElement("org.apache.xpath.XPathFactory");
-                }
-                tests.addElement("org.ietf.jgss.Oid");
-                tests.addElement("org.w3c.dom.Attr");
-                tests.addElement("org.xml.sax.XMLReader");
-                // fall through
-            case VERSION_1_3:
-                tests.addElement("org.omg.CORBA.Any");
-                tests.addElement("com.sun.corba.se.internal.corba.AnyImpl");
-                tests.addElement("com.sun.jndi.ldap.LdapURL");
-                tests.addElement("com.sun.media.sound.Printer");
-                tests.addElement("com.sun.naming.internal.VersionHelper");
-                tests.addElement("com.sun.org.omg.CORBA.Initializer");
-                tests.addElement("sunw.io.Serializable");
-                tests.addElement("sunw.util.EventListener");
-                // fall through
-            case VERSION_1_2:
-                tests.addElement("javax.accessibility.Accessible");
-                tests.addElement("sun.misc.BASE64Encoder");
-                tests.addElement("com.sun.image.codec.jpeg.JPEGCodec");
-                // fall through
-            case VERSION_1_1:
-            default:
-                //things like sun.reflection, sun.misc, sun.net
-                tests.addElement("sun.reflect.SerializationConstructorAccessorImpl");
-                tests.addElement("sun.net.www.http.HttpClient");
-                tests.addElement("sun.audio.AudioPlayer");
-                break;
+        if (isAtLeastJavaVersion(JAVA_1_1)) {
+            //things like sun.reflection, sun.misc, sun.net
+            tests.addElement("sun.reflect.SerializationConstructorAccessorImpl");
+            tests.addElement("sun.net.www.http.HttpClient");
+            tests.addElement("sun.audio.AudioPlayer");
+        }
+        if (isAtLeastJavaVersion(JAVA_1_2)) {
+            tests.addElement("javax.accessibility.Accessible");
+            tests.addElement("sun.misc.BASE64Encoder");
+            tests.addElement("com.sun.image.codec.jpeg.JPEGCodec");
+        }
+        if (isAtLeastJavaVersion(JAVA_1_3)) {
+            tests.addElement("org.omg.CORBA.Any");
+            tests.addElement("com.sun.corba.se.internal.corba.AnyImpl");
+            tests.addElement("com.sun.jndi.ldap.LdapURL");
+            tests.addElement("com.sun.media.sound.Printer");
+            tests.addElement("com.sun.naming.internal.VersionHelper");
+            tests.addElement("com.sun.org.omg.CORBA.Initializer");
+            tests.addElement("sunw.io.Serializable");
+            tests.addElement("sunw.util.EventListener");
+        }
+        if (isAtLeastJavaVersion(JAVA_1_4)) {
+            tests.addElement("sun.audio.AudioPlayer");
+            if (javaVersionNumber == VERSION_1_4) {
+                // only for 1.4, not for higher versions
+                tests.addElement("org.apache.crimson.parser.ContentModel");
+                tests.addElement("org.apache.xalan.processor.ProcessorImport");
+                tests.addElement("org.apache.xml.utils.URI");
+                tests.addElement("org.apache.xpath.XPathFactory");
+            }
+            tests.addElement("org.ietf.jgss.Oid");
+            tests.addElement("org.w3c.dom.Attr");
+            tests.addElement("org.xml.sax.XMLReader");
+        }
+        if (isAtLeastJavaVersion(JAVA_1_5)) {
+            tests.addElement("com.sun.org.apache.xerces.internal.jaxp.datatype.DatatypeFactoryImpl");
+        }
+        if (isAtLeastJavaVersion(JAVA_1_7)) {
+            tests.addElement("jdk.net.Sockets");
         }
         return tests;
     }
@@ -566,26 +567,21 @@ public final class JavaEnvUtils {
     }
 
     /**
-     *
      * Writes the command into a temporary DCL script and returns the
      * corresponding File object.
      * It is the job of the caller to delete the file on exit.
-     * @param cmd the command.
+     * @param cmds the command.
      * @return the file containing the command.
      * @throws IOException if there is an error writing to the file.
      */
-    public static File createVmsJavaOptionFile(String[] cmd)
+    public static File createVmsJavaOptionFile(String[] cmds)
             throws IOException {
         File script = FILE_UTILS.createTempFile("ANT", ".JAVA_OPTS", null, false, true);
-        BufferedWriter out = null;
-        try {
-            out = new BufferedWriter(new FileWriter(script));
-            for (int i = 0; i < cmd.length; i++) {
-                out.write(cmd[i]);
+        try (BufferedWriter out = new BufferedWriter(new FileWriter(script))) {
+            for (String cmd : cmds) {
+                out.write(cmd);
                 out.newLine();
             }
-        } finally {
-            FileUtils.close(out);
         }
         return script;
     }
