@@ -34,89 +34,85 @@ import java.io.OutputStream;
  * <p>
  * The compression requires large amounts of memory. Thus you should call the
  * {@link #close() close()} method as soon as possible, to force
- * <tt>CBZip2OutputStream</tt> to release the allocated memory.
+ * <code>CBZip2OutputStream</code> to release the allocated memory.
  * </p>
  *
- * <p> You can shrink the amount of allocated memory and maybe raise
+ * <p>You can shrink the amount of allocated memory and maybe raise
  * the compression speed by choosing a lower blocksize, which in turn
  * may cause a lower compression ratio. You can avoid unnecessary
  * memory allocation by avoiding using a blocksize which is bigger
- * than the size of the input.  </p>
+ * than the size of the input.</p>
  *
- * <p> You can compute the memory usage for compressing by the
- * following formula: </p>
+ * <p>You can compute the memory usage for compressing by the
+ * following formula:</p>
  *
  * <pre>
  * &lt;code&gt;400k + (9 * blocksize)&lt;/code&gt;.
  * </pre>
  *
- * <p> To get the memory required for decompression by {@link
- * CBZip2InputStream CBZip2InputStream} use </p>
+ * <p>To get the memory required for decompression by {@link
+ * CBZip2InputStream CBZip2InputStream} use</p>
  *
  * <pre>
  * &lt;code&gt;65k + (5 * blocksize)&lt;/code&gt;.
  * </pre>
  *
- * <table width="100%" border="1">
- * <colgroup> <col width="33%" /> <col width="33%" /> <col width="33%" />
- * </colgroup>
+ * <table style="border:1px solid black">
+ * <caption>Memory usage by blocksize</caption>
  * <tr>
- * <th colspan="3">Memory usage by blocksize</th>
+ * <th style="text-align:right">Blocksize</th>
+ * <th style="text-align:right">Compression<br>memory usage</th>
+ * <th style="text-align:right">Decompression<br>memory usage</th>
  * </tr>
  * <tr>
- * <th align="right">Blocksize</th> <th align="right">Compression<br>
- * memory usage</th> <th align="right">Decompression<br>
- * memory usage</th>
+ * <td style="text-align:right">100k</td>
+ * <td style="text-align:right">1300k</td>
+ * <td style="text-align:right">565k</td>
  * </tr>
  * <tr>
- * <td align="right">100k</td>
- * <td align="right">1300k</td>
- * <td align="right">565k</td>
+ * <td style="text-align:right">200k</td>
+ * <td style="text-align:right">2200k</td>
+ * <td style="text-align:right">1065k</td>
  * </tr>
  * <tr>
- * <td align="right">200k</td>
- * <td align="right">2200k</td>
- * <td align="right">1065k</td>
+ * <td style="text-align:right">300k</td>
+ * <td style="text-align:right">3100k</td>
+ * <td style="text-align:right">1565k</td>
  * </tr>
  * <tr>
- * <td align="right">300k</td>
- * <td align="right">3100k</td>
- * <td align="right">1565k</td>
+ * <td style="text-align:right">400k</td>
+ * <td style="text-align:right">4000k</td>
+ * <td style="text-align:right">2065k</td>
  * </tr>
  * <tr>
- * <td align="right">400k</td>
- * <td align="right">4000k</td>
- * <td align="right">2065k</td>
+ * <td style="text-align:right">500k</td>
+ * <td style="text-align:right">4900k</td>
+ * <td style="text-align:right">2565k</td>
  * </tr>
  * <tr>
- * <td align="right">500k</td>
- * <td align="right">4900k</td>
- * <td align="right">2565k</td>
+ * <td style="text-align:right">600k</td>
+ * <td style="text-align:right">5800k</td>
+ * <td style="text-align:right">3065k</td>
  * </tr>
  * <tr>
- * <td align="right">600k</td>
- * <td align="right">5800k</td>
- * <td align="right">3065k</td>
+ * <td style="text-align:right">700k</td>
+ * <td style="text-align:right">6700k</td>
+ * <td style="text-align:right">3565k</td>
  * </tr>
  * <tr>
- * <td align="right">700k</td>
- * <td align="right">6700k</td>
- * <td align="right">3565k</td>
+ * <td style="text-align:right">800k</td>
+ * <td style="text-align:right">7600k</td>
+ * <td style="text-align:right">4065k</td>
  * </tr>
  * <tr>
- * <td align="right">800k</td>
- * <td align="right">7600k</td>
- * <td align="right">4065k</td>
- * </tr>
- * <tr>
- * <td align="right">900k</td>
- * <td align="right">8500k</td>
- * <td align="right">4565k</td>
+ * <td style="text-align:right">900k</td>
+ * <td style="text-align:right">8500k</td>
+ * <td style="text-align:right">4565k</td>
  * </tr>
  * </table>
  *
  * <p>
- * For decompression <tt>CBZip2InputStream</tt> allocates less memory if the
+ * For decompression <code>CBZip2InputStream</code> allocates less memory if the
  * bzipped input is smaller than one block.
  * </p>
  *
@@ -133,12 +129,12 @@ public class CBZip2OutputStream extends OutputStream
     implements BZip2Constants {
 
     /**
-     * The minimum supported blocksize <tt> == 1</tt>.
+     * The minimum supported blocksize <code> == 1</code>.
      */
     public static final int MIN_BLOCKSIZE = 1;
 
     /**
-     * The maximum supported blocksize <tt> == 9</tt>.
+     * The maximum supported blocksize <code> == 9</code>.
      */
     public static final int MAX_BLOCKSIZE = 9;
 
@@ -195,10 +191,10 @@ public class CBZip2OutputStream extends OutputStream
      * This constant is accessible by subclasses for historical
      * purposes. If you don't know what it means then you don't need
      * it.
-     * <p> If you are ever unlucky/improbable enough to get a stack
+     * <p>If you are ever unlucky/improbable enough to get a stack
      * overflow whilst sorting, increase the following constant and
      * try again. In practice I have never seen the stack go above 27
-     * elems, so the following limit seems very generous.  </p>
+     * elems, so the following limit seems very generous.</p>
      */
     protected static final int QSORT_STACK_SIZE = 1000;
 
@@ -207,14 +203,20 @@ public class CBZip2OutputStream extends OutputStream
      * Possibly because the number of elems to sort is usually small, typically
      * &lt;= 20.
      */
-    private static final int[] INCS = { 1, 4, 13, 40, 121, 364, 1093, 3280,
-                                        9841, 29524, 88573, 265720, 797161,
-                                        2391484 };
+    @SuppressWarnings("unused")
+    private static final int[] INCS = {1, 4, 13, 40, 121, 364, 1093, 3280,
+                                       9841, 29524, 88573, 265720, 797161,
+                                       2391484};
 
     /**
      * This method is accessible by subclasses for historical
      * purposes. If you don't know what it does then you don't need
      * it.
+     *
+     * @param len char[]
+     * @param freq char[]
+     * @param alphaSize int
+     * @param maxLen int
      */
     protected static void hbMakeCodeLengths(char[] len, int[] freq,
                                             int alphaSize, int maxLen) {
@@ -560,14 +562,13 @@ public class CBZip2OutputStream extends OutputStream
     /**
      * Chooses a blocksize based on the given length of the data to compress.
      *
-     * @return The blocksize, between {@link #MIN_BLOCKSIZE} and
-     *         {@link #MAX_BLOCKSIZE} both inclusive. For a negative
-     *         <tt>inputLength</tt> this method returns <tt>MAX_BLOCKSIZE</tt>
-     *         always.
-     *
      * @param inputLength
      *            The length of the data which will be compressed by
-     *            <tt>CBZip2OutputStream</tt>.
+     *            <code>CBZip2OutputStream</code>.
+     * @return The blocksize, between {@link #MIN_BLOCKSIZE} and
+     *         {@link #MAX_BLOCKSIZE} both inclusive. For a negative
+     *         <code>inputLength</code> this method returns <code>MAX_BLOCKSIZE</code>
+     *         always.
      */
     public static int chooseBlockSize(long inputLength) {
         return (inputLength > 0) ? (int) Math
@@ -575,11 +576,11 @@ public class CBZip2OutputStream extends OutputStream
     }
 
     /**
-     * Constructs a new <tt>CBZip2OutputStream</tt> with a blocksize of 900k.
+     * Constructs a new <code>CBZip2OutputStream</code> with a blocksize of 900k.
      *
      * <p>
      * <b>Attention: </b>The caller is responsible to write the two BZip2 magic
-     * bytes <tt>"BZ"</tt> to the specified stream prior to calling this
+     * bytes <code>"BZ"</code> to the specified stream prior to calling this
      * constructor.
      * </p>
      *
@@ -596,11 +597,11 @@ public class CBZip2OutputStream extends OutputStream
     }
 
     /**
-     * Constructs a new <tt>CBZip2OutputStream</tt> with specified blocksize.
+     * Constructs a new <code>CBZip2OutputStream</code> with specified blocksize.
      *
      * <p>
      * <b>Attention: </b>The caller is responsible to write the two BZip2 magic
-     * bytes <tt>"BZ"</tt> to the specified stream prior to calling this
+     * bytes <code>"BZ"</code> to the specified stream prior to calling this
      * constructor.
      * </p>
      *
@@ -675,41 +676,34 @@ public class CBZip2OutputStream extends OutputStream
 
             int runLengthShadow = this.runLength;
             this.crc.updateCRC(currentCharShadow, runLengthShadow);
+            final byte[] block = dataShadow.block;
 
             switch (runLengthShadow) {
-            case 1:
-                dataShadow.block[lastShadow + 2] = ch;
-                this.last = lastShadow + 1;
-                break;
-
-            case 2:
-                dataShadow.block[lastShadow + 2] = ch;
-                dataShadow.block[lastShadow + 3] = ch;
-                this.last = lastShadow + 2;
-                break;
-
-            case 3: {
-                final byte[] block = dataShadow.block;
-                block[lastShadow + 2] = ch;
-                block[lastShadow + 3] = ch;
-                block[lastShadow + 4] = ch;
-                this.last = lastShadow + 3;
-            }
-                break;
-
-            default: {
-                runLengthShadow -= 4;
-                dataShadow.inUse[runLengthShadow] = true;
-                final byte[] block = dataShadow.block;
-                block[lastShadow + 2] = ch;
-                block[lastShadow + 3] = ch;
-                block[lastShadow + 4] = ch;
-                block[lastShadow + 5] = ch;
-                block[lastShadow + 6] = (byte) runLengthShadow;
-                this.last = lastShadow + 5;
-            }
-                break;
-
+                case 1:
+                    block[lastShadow + 2] = ch;
+                    this.last = lastShadow + 1;
+                    break;
+                case 2:
+                    block[lastShadow + 2] = ch;
+                    block[lastShadow + 3] = ch;
+                    this.last = lastShadow + 2;
+                    break;
+                case 3:
+                    block[lastShadow + 2] = ch;
+                    block[lastShadow + 3] = ch;
+                    block[lastShadow + 4] = ch;
+                    this.last = lastShadow + 3;
+                    break;
+                default:
+                    runLengthShadow -= 4;
+                    dataShadow.inUse[runLengthShadow] = true;
+                    block[lastShadow + 2] = ch;
+                    block[lastShadow + 3] = ch;
+                    block[lastShadow + 4] = ch;
+                    block[lastShadow + 5] = ch;
+                    block[lastShadow + 6] = (byte) runLengthShadow;
+                    this.last = lastShadow + 5;
+                    break;
             }
         } else {
             endBlock();
@@ -814,7 +808,7 @@ public class CBZip2OutputStream extends OutputStream
          * is about 2.0e-3 for 32 bits, 1.0e-5 for 40 bits and 4.0e-8 for 48
          * bits. For a compressed file of size 100Gb -- about 100000 blocks --
          * only a 48-bit marker will do. NB: normal compression/ decompression
-         * donot rely on these statistical properties. They are only important
+         * do not rely on these statistical properties. They are only important
          * when trying to recover blocks from damaged files.
          */
         bsPutUByte(0x31);
@@ -854,6 +848,8 @@ public class CBZip2OutputStream extends OutputStream
 
     /**
      * Returns the blocksize parameter specified at construction time.
+     *
+     * @return int
      */
     public final int getBlockSize() {
         return this.blockSize100k;
@@ -1012,18 +1008,17 @@ public class CBZip2OutputStream extends OutputStream
             int ge = gs - 1;
             int aFreq = 0;
 
-            for (final int a = alphaSize - 1; (aFreq < tFreq) && (ge < a);) {
+            while (aFreq < tFreq && ge < alphaSize - 1) {
                 aFreq += mtfFreq[++ge];
             }
 
-            if ((ge > gs) && (nPart != nGroups) && (nPart != 1)
-                && (((nGroups - nPart) & 1) != 0)) {
+            if (ge > gs && nPart != nGroups && nPart != 1 && (nGroups - nPart & 1) != 0) {
                 aFreq -= mtfFreq[ge--];
             }
 
             final byte[] len_np = len[nPart - 1];
             for (int v = alphaSize; --v >= 0;) {
-                if ((v >= gs) && (v <= ge)) {
+                if (v >= gs && v <= ge) {
                     len_np[v] = LESSER_ICOST;
                 } else {
                     len_np[v] = GREATER_ICOST;
@@ -1517,7 +1512,7 @@ public class CBZip2OutputStream extends OutputStream
         this.nMTF = wr + 1;
     }
 
-    static final class Data extends Object {
+    static final class Data {
 
         // with blockSize 900k
         /* maps unsigned byte => "does it occur in block" */
