@@ -28,6 +28,7 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.io.StringWriter;
 import java.nio.ByteBuffer;
+import java.nio.charset.StandardCharsets;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.Map;
@@ -94,6 +95,7 @@ public class TarOutputStream extends FilterOutputStream {
 
     /**
      * Constructor for TarInputStream.
+     *
      * @param os the output stream to use
      */
     public TarOutputStream(OutputStream os) {
@@ -102,6 +104,7 @@ public class TarOutputStream extends FilterOutputStream {
 
     /**
      * Constructor for TarInputStream.
+     *
      * @param os the output stream to use
      * @param encoding name of the encoding to use for file names
      */
@@ -111,6 +114,7 @@ public class TarOutputStream extends FilterOutputStream {
 
     /**
      * Constructor for TarInputStream.
+     *
      * @param os the output stream to use
      * @param blockSize the block size to use
      */
@@ -120,6 +124,7 @@ public class TarOutputStream extends FilterOutputStream {
 
     /**
      * Constructor for TarInputStream.
+     *
      * @param os the output stream to use
      * @param blockSize the block size to use
      * @param encoding name of the encoding to use for file names
@@ -130,6 +135,7 @@ public class TarOutputStream extends FilterOutputStream {
 
     /**
      * Constructor for TarInputStream.
+     *
      * @param os the output stream to use
      * @param blockSize the block size to use
      * @param recordSize the record size to use
@@ -140,6 +146,7 @@ public class TarOutputStream extends FilterOutputStream {
 
     /**
      * Constructor for TarInputStream.
+     *
      * @param os the output stream to use
      * @param blockSize the block size to use
      * @param recordSize the record size to use
@@ -163,6 +170,7 @@ public class TarOutputStream extends FilterOutputStream {
      * This can be LONGFILE_ERROR(0), LONGFILE_TRUNCATE(1) or LONGFILE_GNU(2).
      * This specifies the treatment of long file names (names &gt;= TarConstants.NAMELEN).
      * Default is LONGFILE_ERROR.
+     *
      * @param longFileMode the mode to use
      */
     public void setLongFileMode(int longFileMode) {
@@ -174,6 +182,7 @@ public class TarOutputStream extends FilterOutputStream {
      * This can be BIGNUMBER_ERROR(0), BIGNUMBER_POSIX(1) or BIGNUMBER_STAR(2).
      * This specifies the treatment of big files (sizes &gt; TarConstants.MAXSIZE) and other numeric values to big to fit into a traditional tar header.
      * Default is BIGNUMBER_ERROR.
+     *
      * @param bigNumberMode the mode to use
      */
     public void setBigNumberMode(int bigNumberMode) {
@@ -182,6 +191,8 @@ public class TarOutputStream extends FilterOutputStream {
 
     /**
      * Whether to add a PAX extension header for non-ASCII file names.
+     *
+     * @param b boolean
      */
     public void setAddPaxHeadersForNonAsciiNames(boolean b) {
         addPaxHeadersForNonAsciiNames = b;
@@ -236,7 +247,7 @@ public class TarOutputStream extends FilterOutputStream {
      */
     @Override
     public void close() throws IOException {
-        if(!finished) {
+        if (!finished) {
             finish();
         }
 
@@ -269,16 +280,16 @@ public class TarOutputStream extends FilterOutputStream {
      * @throws IOException on error
      */
     public void putNextEntry(TarEntry entry) throws IOException {
-        if(finished) {
+        if (finished) {
             throw new IOException("Stream has already been finished");
         }
-        Map<String, String> paxHeaders = new HashMap<String, String>();
+        Map<String, String> paxHeaders = new HashMap<>();
         final String entryName = entry.getName();
         boolean paxHeaderContainsPath = handleLongName(entry, entryName, paxHeaders, "path",
                                                        TarConstants.LF_GNUTYPE_LONGNAME, "file name");
 
         final String linkName = entry.getLinkName();
-        boolean paxHeaderContainsLinkPath = linkName != null && linkName.length() > 0
+        boolean paxHeaderContainsLinkPath = linkName != null && !linkName.isEmpty()
             && handleLongName(entry, linkName, paxHeaders, "linkpath",
                               TarConstants.LF_GNUTYPE_LONGLINK, "link name");
 
@@ -332,7 +343,7 @@ public class TarOutputStream extends FilterOutputStream {
         if (finished) {
             throw new IOException("Stream has already been finished");
         }
-        if (!haveUnclosedEntry){
+        if (!haveUnclosedEntry) {
             throw new IOException("No current entry to close");
         }
         if (assemLen > 0) {
@@ -358,7 +369,7 @@ public class TarOutputStream extends FilterOutputStream {
     /**
      * Writes a byte to the current tar archive entry.
      *
-     * This method simply calls read( byte[], int, int ).
+     * This method simply calls read(byte[], int, int).
      *
      * @param b The byte written.
      * @throws IOException on error
@@ -373,7 +384,7 @@ public class TarOutputStream extends FilterOutputStream {
     /**
      * Writes bytes to the current tar archive entry.
      *
-     * This method simply calls write( byte[], int, int ).
+     * This method simply calls write(byte[], int, int).
      *
      * @param wBuf The buffer to write to the archive.
      * @throws IOException on error
@@ -490,7 +501,7 @@ public class TarOutputStream extends FilterOutputStream {
                 + 3 /* blank, equals and newline */
                 + 2 /* guess 9 < actual length < 100 */;
             String line = len + " " + key + "=" + value + "\n";
-            int actualLength = line.getBytes("UTF-8").length;
+            int actualLength = line.getBytes(StandardCharsets.UTF_8).length;
             while (len != actualLength) {
                 // Adjust for cases where length < 10 or > 100
                 // or where UTF-8 encoding isn't a single octet
@@ -499,11 +510,11 @@ public class TarOutputStream extends FilterOutputStream {
                 // first pass so we'd need a second.
                 len = actualLength;
                 line = len + " " + key + "=" + value + "\n";
-                actualLength = line.getBytes("UTF-8").length;
+                actualLength = line.getBytes(StandardCharsets.UTF_8).length;
             }
             w.write(line);
         }
-        byte[] data = w.toString().getBytes("UTF-8");
+        byte[] data = w.toString().getBytes(StandardCharsets.UTF_8);
         pex.setSize(data.length);
         putNextEntry(pex);
         write(data);
@@ -511,10 +522,9 @@ public class TarOutputStream extends FilterOutputStream {
     }
 
     private String stripTo7Bits(String name) {
-        final int length = name.length();
-        StringBuilder result = new StringBuilder(length);
-        for (int i = 0; i < length; i++) {
-            char stripped = (char) (name.charAt(i) & 0x7F);
+        StringBuilder result = new StringBuilder(name.length());
+        for (final char ch : name.toCharArray()) {
+            char stripped = (char) (ch & 0x7F);
             if (stripped != 0) { // would be read as Trailing null
                 result.append(stripped);
             }
@@ -614,7 +624,7 @@ public class TarOutputStream extends FilterOutputStream {
      * @param fieldName the name of the field
      * @return whether a pax header has been written.
      */
-    private boolean handleLongName(TarEntry entry , String name,
+    private boolean handleLongName(TarEntry entry, String name,
                                    Map<String, String> paxHeaders,
                                    String paxHeaderName, byte linkType, String fieldName)
         throws IOException {
