@@ -18,57 +18,61 @@
 
 package org.apache.tools.ant.taskdefs;
 
-import org.apache.tools.ant.*;
-import org.apache.tools.ant.util.FileUtils;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assume.assumeNotNull;
 
 import java.io.File;
 import java.util.GregorianCalendar;
 
+import org.apache.tools.ant.BuildEvent;
+import org.apache.tools.ant.BuildFileRule;
+import org.apache.tools.ant.BuildListener;
+import org.apache.tools.ant.MagicNames;
+import org.apache.tools.ant.Project;
+import org.apache.tools.ant.ProjectHelper;
+import org.apache.tools.ant.util.FileUtils;
+import org.junit.Before;
+import org.junit.Ignore;
+import org.junit.Rule;
+import org.junit.Test;
+
 /**
  * Unit test for the &lt;exec&gt; task.
  */
-public class ExecTaskTest extends BuildFileTest {
-    private static final String BUILD_PATH = "src/etc/testcases/taskdefs/exec/";
-    private static final String BUILD_FILE = BUILD_PATH + "exec.xml";
+public class ExecTaskTest {
+
+    @Rule
+    public BuildFileRule buildRule = new BuildFileRule();
+
     private static final int TIME_TO_WAIT = 1;
     /** maximum time allowed for the build in milliseconds */
-    private static final int MAX_BUILD_TIME = 4000;
-    private static final int SECURITY_MARGIN = 2000; // wait 2 second extras
-    // the test failed with 100 ms of margin on cvs.apache.org on August 1st,
-    // 2003
+    private static final int MAX_BUILD_TIME = 6000;
+    private static final int SECURITY_MARGIN = 4000; // wait 4 second extras
+    // the test failed with 100 ms of margin on cvs.apache.org on August 1st, 2003
+    // the test randomly failed with 3 s of margin on Windows Jenkins slaves on during July 2014
 
     /** Utilities used for file operations */
     private static final FileUtils FILE_UTILS = FileUtils.getFileUtils();
 
-    private File logFile;
-    private MonitoredBuild myBuild = null;
-    volatile private boolean buildFinished = false;
-
-    public ExecTaskTest(String name) {
-        super(name);
-    }
+    private volatile boolean buildFinished = false;
 
+    @Before
     public void setUp() {
-        configureProject(BUILD_FILE);
+        buildRule.configureProject("src/etc/testcases/taskdefs/exec/exec.xml");
     }
 
-    public void tearDown() {
-        if (logFile != null && logFile.exists()) {
-            getProject().setProperty("logFile", logFile.getAbsolutePath());
-        }
-        executeTarget("cleanup");
-    }
-
-    public void testspawn() {
-        project.executeTarget("init");
-        if (project.getProperty("test.can.run") == null) {
-            return;
-        }
-        myBuild = new MonitoredBuild(new File(System.getProperty("root"), BUILD_FILE), "spawn");
-        logFile = FILE_UTILS.createTempFile("spawn", "log", project.getBaseDir(), false, false);
+    @Test
+    public void testspawn() throws InterruptedException {
+        buildRule.getProject().executeTarget("setUp");
+        assumeNotNull(buildRule.getProject().getProperty("test.can.run"));
+        MonitoredBuild myBuild = new MonitoredBuild(new File(
+                buildRule.getProject().getProperty(MagicNames.ANT_FILE)), "spawn");
+        File logFile = FILE_UTILS.createTempFile("spawn", "log",
+                new File(buildRule.getProject().getProperty("output")), false, false);
         // this is guaranteed by FileUtils#createTempFile
-        assertTrue("log file not existing", !logFile.exists());
-        // make the spawned process run 4 seconds
+        assertFalse("log file not existing", logFile.exists());
+        // make the spawned process run 1 seconds
         myBuild.setTimeToWait(TIME_TO_WAIT);
         myBuild.setLogFile(logFile.getAbsolutePath());
         myBuild.addBuildListener(new MonitoredBuildListener());
@@ -76,11 +80,7 @@ public class ExecTaskTest extends BuildFileTest {
         GregorianCalendar startwait = new GregorianCalendar();
         // this loop runs parallel to the build
         while (!buildFinished) {
-            try {
-                Thread.sleep(10);
-            } catch (InterruptedException e) {
-                System.out.println("my sleep was interrupted");
-            }
+            Thread.sleep(10);
             GregorianCalendar now = new GregorianCalendar();
             // security
             if (now.getTime().getTime() - startwait.getTime().getTime() > MAX_BUILD_TIME) {
@@ -91,11 +91,7 @@ public class ExecTaskTest extends BuildFileTest {
             }
         }
         // now wait until the spawned process is finished
-        try {
-            Thread.sleep((TIME_TO_WAIT) * 1000 + SECURITY_MARGIN);
-        } catch (InterruptedException e) {
-            System.out.println("my sleep was interrupted");
-        }
+        Thread.sleep((TIME_TO_WAIT) * 1000 + SECURITY_MARGIN);
         // time of the build in milli seconds
         long elapsed = myBuild.getTimeElapsed();
         assertTrue("we waited more than the process lasted",
@@ -103,9 +99,12 @@ public class ExecTaskTest extends BuildFileTest {
         logFile = new File(logFile.getAbsolutePath());
         assertTrue("log file found after spawn", logFile.exists());
     }
-    
+
+    @Test
+    @Ignore("#50507 - fails at least on Linux")
+    /* TODO #50507 - fails at least on Linux */
     public void testOutAndErr() {
-        project.executeTarget("test-out-and-err");
+        buildRule.getProject().executeTarget("test-out-and-err");
     }
 
     private static class MonitoredBuild implements Runnable {
@@ -113,18 +112,14 @@ public class ExecTaskTest extends BuildFileTest {
         private File myBuildFile = null;
         private String target = null;
         private Project project = null;
-        private int timeToWait = 0;
-        private String logFile = null;
         private GregorianCalendar timeStarted = null;
         private GregorianCalendar timeFinished = null;
 
         public void setLogFile(String logFile) {
-            this.logFile = logFile;
             project.setProperty("logFile", logFile);
         }
 
         public void setTimeToWait(int timeToWait) {
-            this.timeToWait = timeToWait;
             project.setProperty("timeToWait", Long.toString(timeToWait));
         }
 
@@ -136,9 +131,8 @@ public class ExecTaskTest extends BuildFileTest {
             myBuildFile = buildFile;
             this.target = target;
             project = new Project();
-            project = new Project();
             project.init();
-            project.setUserProperty("ant.file", myBuildFile.getAbsolutePath());
+            project.setUserProperty(MagicNames.ANT_FILE, myBuildFile.getAbsolutePath());
             ProjectHelper.configureProject(project, myBuildFile);
         }
 
