@@ -17,6 +17,7 @@
  */
 package org.apache.tools.ant.util;
 
+import org.apache.tools.ant.MagicTestNames;
 import org.junit.Test;
 
 import java.io.ByteArrayOutputStream;
@@ -24,15 +25,17 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.InputStreamReader;
 import java.io.StringReader;
+import java.nio.charset.StandardCharsets;
 
 import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.fail;
 
 /**
  * Test for ReaderInputStream
  */
 public class ReaderInputStreamTest {
 
+    private static final String ROOT = System.getProperty(MagicTestNames.TEST_ROOT_DIRECTORY);
+
     @Test
     public void testSimple() throws Exception {
         compareBytes("abc", "utf-8");
@@ -59,9 +62,9 @@ public class ReaderInputStreamTest {
             }
             bytes[pos++] = (byte) res;
         }
-        bytes = "abc".getBytes("utf-16");
+        bytes = "abc".getBytes(StandardCharsets.UTF_16);
         //        String n = new String(bytes, 0, pos, "utf-16");
-        new String(bytes, 0, bytes.length, "utf-16");
+        new String(bytes, 0, bytes.length, StandardCharsets.UTF_16);
     }
 
     @SuppressWarnings("resource")
@@ -79,69 +82,54 @@ public class ReaderInputStreamTest {
     }
 
     @Test
-    public void testPreample() throws Exception {
-        byte[] bytes = "".getBytes("utf-16");
+    public void testPreample() {
+        byte[] bytes = "".getBytes(StandardCharsets.UTF_16);
         System.out.println("Preample len is " + bytes.length);
     }
 
     @Test
     public void testIso88591ToUtf8() throws Exception {
-        InputStreamReader fin = null;
-        ReaderInputStream r = null;
-        FileInputStream utf8 = null;
-        try {
-            fin = new InputStreamReader(new FileInputStream(new File(System.getProperty("root"),
-                    "src/tests/antunit/taskdefs/exec/input/iso8859-1")),
-                                        "ISO8859_1");
-            r = new ReaderInputStream(fin, "UTF8");
-
-            ByteArrayOutputStream actualOS = new ByteArrayOutputStream();
+        ByteArrayOutputStream actualOS = new ByteArrayOutputStream();
+        try (ReaderInputStream r = new ReaderInputStream(new InputStreamReader(new FileInputStream(new File(ROOT,
+                "src/tests/antunit/taskdefs/exec/input/iso8859-1")), StandardCharsets.ISO_8859_1),
+                StandardCharsets.UTF_8)) {
             int b = r.read();
             while (b > -1) {
                 actualOS.write((byte) b);
                 b = r.read();
             }
+        }
 
-            utf8 = new FileInputStream(new File(System.getProperty("root"),
-                    "src/tests/antunit/taskdefs/exec/expected/utf-8"));
-            ByteArrayOutputStream expectedOS = new ByteArrayOutputStream();
-            b = utf8.read();
+        ByteArrayOutputStream expectedOS = new ByteArrayOutputStream();
+        try (FileInputStream utf8 = new FileInputStream(new File(ROOT,
+                "src/tests/antunit/taskdefs/exec/expected/utf-8"))) {
+            int b = utf8.read();
             while (b > -1) {
                 expectedOS.write((byte) b);
                 b = utf8.read();
             }
+        }
 
-            byte[] expected = expectedOS.toByteArray();
-            byte[] actual = actualOS.toByteArray();
-            assertEquals("length", expected.length, actual.length);
-            for (int i = 0; i < actual.length; i++) {
-                assertEquals("byte " + i, expected[i], actual[i]);
-            }
-        } finally {
-            FileUtils.close(fin);
-            FileUtils.close(r);
-            FileUtils.close(utf8);
+        byte[] expected = expectedOS.toByteArray();
+        byte[] actual = actualOS.toByteArray();
+        assertEquals("length", expected.length, actual.length);
+        for (int i = 0; i < actual.length; i++) {
+            assertEquals("byte " + i, expected[i], actual[i]);
         }
     }
 
     @SuppressWarnings("resource")
     private void compareBytes(String s, String encoding) throws Exception {
         byte[] expected = s.getBytes(encoding);
-
-        ReaderInputStream r = new ReaderInputStream(
-            new StringReader(s), encoding);
+        ReaderInputStream r = new ReaderInputStream(new StringReader(s), encoding);
         for (int i = 0; i < expected.length; ++i) {
             int expect = expected[i] & 0xFF;
             int read = r.read();
-            if (expect != read) {
-                fail("Mismatch in ReaderInputStream at index " + i
-                     + " expecting " + expect + " got " + read + " for string "
-                     + s + " with encoding " + encoding);
-            }
-        }
-        if (r.read() != -1) {
-            fail("Mismatch in ReaderInputStream - EOF not seen for string "
-                 + s + " with encoding " + encoding);
+            assertEquals("Mismatch in ReaderInputStream at index " + i
+                    + " expecting " + expect + " got " + read + " for string "
+                    + s + " with encoding " + encoding, expect, read);
         }
+        assertEquals("Mismatch in ReaderInputStream - EOF not seen for string "
+                + s + " with encoding " + encoding, -1, r.read());
     }
 }
